// include gulp
var gulp = require('gulp');

// include plug-ins
var argv = require('yargs').argv;
var jshint = require('gulp-jshint');
var minifyHTML = require('gulp-minify-html');
var concat = require('gulp-concat');
var stripDebug = require('gulp-strip-debug');
var uglify = require('gulp-uglify');
var clean = require('gulp-clean');
var ngAnnotate = require('gulp-ng-annotate');
var minifyCSS = require('gulp-minify-css');
var runSequence = require('run-sequence');
var preprocess = require('gulp-preprocess');
var gulpNgConfig = require('gulp-ng-config');
var rename = require('gulp-rename');
var cache = require('gulp-cached');
var karma = require('karma').server;

gulp.task('jshint', function() {
  return gulp.src(['server.js','./public/*.js','./public/*/*.js', '!./public/vendor/*', './test/*.js', './test/*/*.js'])
    .pipe(jshint())
    .pipe(jshint.reporter('default'))
    .pipe(jshint.reporter('fail'));
});

gulp.task('test', function (done) {
  runSequence('js')
  karma.start({
    configFile: __dirname + '/karma.conf.js',
    singleRun: true
  }, done);
});

gulp.task('prod-config', function () {
  return gulp.src('prod-config.json')
        .pipe(gulpNgConfig('cmb.config'))
        .pipe(rename("config.js"))
        .pipe(gulp.dest('./public/config'));
});

gulp.task('dev-config', function () {
  return gulp.src('dev-config.json')
        .pipe(gulpNgConfig('cmb.config'))
        .pipe(rename("config.js"))
        .pipe(gulp.dest('./public/config'));
});

gulp.task('local-config', function () {
  return gulp.src('local-config.json')
        .pipe(gulpNgConfig('cmb.config'))
        .pipe(rename("config.js"))
        .pipe(gulp.dest('./public/config'));
});

gulp.task('clean', function () {
  return gulp.src('build', {read: false})
    .pipe(clean());
});

gulp.task('html', function() {
  var htmlDst = './build';

  gulp.src('./public/index.html')
  	.pipe(preprocess({context: {ENV: 'dev'}}))
    .pipe(gulp.dest(htmlDst));

  gulp.src('./public/partials/*.html')
    .pipe(gulp.dest(htmlDst+'/partials'));

  gulp.src('./public/templates/*.html')
    .pipe(gulp.dest(htmlDst+'/templates'));
});

gulp.task('minify-html', function() {
  var htmlDst = './build';

  gulp.src('./public/index.html')
    .pipe(preprocess({context: {ENV: 'prod'}}))
    .pipe(gulp.dest(htmlDst));

  gulp.src('./public/partials/*.html')
    .pipe(gulp.dest(htmlDst+'/partials'));

  gulp.src('./public/templates/*.html')
    .pipe(gulp.dest(htmlDst+'/templates'));
});

gulp.task('maintenance-html', function() {
  gulp.src('./public/maintenance/index.html')
    .pipe(gulp.dest('./build'));
});

gulp.task('js', function () {
  gulp.src(['public/*.js', 'public/*/*.js'])
    .pipe(gulp.dest('build/scripts'));
});

gulp.task('minify-js', function () {
  gulp.src(['public/*.js', 'public/*/*.js'])
    .pipe(stripDebug())
    .pipe(concat('app.js'))
    .pipe(ngAnnotate())
    .pipe(uglify())
    .pipe(gulp.dest('build/scripts'));
});

gulp.task('css', function() {
  gulp.src('./public/css/*.css')
    .pipe(gulp.dest('./build/css'))
});

gulp.task('minify-css', function() {
  gulp.src('./public/css/*.css')
    .pipe(minifyCSS())
    .pipe(gulp.dest('./build/css'))
});

gulp.task('watch', function (){
  gulp.watch('./public/css/*.css', ['css']);
  gulp.watch(['public/*.js', 'public/*/*.js', 'test/**/*.js'], ['test']);
  gulp.watch(['./public/index.html','./public/partials/*.html', './public/templates/*.html'], ['html']);
});

gulp.task('dev', function (){
  runSequence('clean','dev-config',['css','html','js']);
});

gulp.task('prod', function () {
	runSequence('clean','prod-config', ['minify-css', 'minify-js','minify-html']);
});

gulp.task('maintenance', function() {
  runSequence('clean','maintenance-html','minify-css');
});

gulp.task('default', function () {
  runSequence('clean','local-config', ['css', 'js','html'], 'watch');
});

gulp.task('local-dev', function() {
  runSequence('clean', 'local-config', ['css', 'js','html']);
});
