angular.module('cmb')
.controller('GroupCtrl', function($scope, $http, $auth, $alert, $state, $stateParams, $analytics, myConfig) {
      'use strict';
      $scope.apiurl = myConfig.apiurl+'/api';
      $scope.member = false;
      $scope.groupState = "";

      $scope.isAuthenticated = function() {
          return $auth.isAuthenticated();
        };

      $scope.get_me = function(){
        $http.get($scope.apiurl+'/me', {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}})
                  .success(function(data){
                    _.forEach($scope.group.members, function(member){
                      if(member.user_id === data.user.id){
                        $scope.member = true;
                      }
                    });
                    if($scope.member){
                      _.forEach($scope.group.problems, function(prob){
                        prob.state = "Not Delivered";
                        prob.stateStyle = {'color':'red'};
                        _.forEach(data.user.submissions, function(sub){
                          if(sub.problem_id === prob.problem_id){
                            if(sub.state === "finished"){
                              prob.stateStyle = {'color':'green'};
                              prob.state = "Success";
                            } else {
                              prob.stateStyle = {'color':'orange'};
                              prob.state = "Submitted";
                            }
                          }
                        });
                      });
                    }
                  })
                  .error(function(error) {
                    var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                    console.log(error);
                    $scope.group_alert = $alert({
                      content: message,
                      container: '.main-content',
                      placement: 'top',
                      type: 'error',
                      duration: 5
                    });
                  });
      };

      $scope.redirect_if_leader = function(){
        $http.get($scope.apiurl+'/leader/'+$stateParams.group_id, {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}})
            .success(function(data) {
              if(data.message === "Leader"){
                $state.go('leader', {'group_id': $stateParams.group_id});
              }
            })
            .error(function(error){
              var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                    console.log(error);
                    $scope.group_alert = $alert({
                      content: message,
                      container: '.main-content',
                      placement: 'top',
                      type: 'error',
                      duration: 5
                    });
          });
        };

      if($auth.isAuthenticated()){
        $scope.redirect_if_leader();
      }

      $http.get($scope.apiurl+'/group/'+$stateParams.group_id)
          .success(function(data) {
              $scope.group = data.group;
              if($scope.group.public){
                $scope.groupState = "Public";
              } else {
                $scope.groupState = "Private";
              }
              if($auth.isAuthenticated()){
                $scope.get_me();
              }
          })
          .error(function(error) {
              var message = ( (error && error.message) ? error.message : "Could not connect to server!");
              console.log(error);
              $scope.group_alert = $alert({
                content: message,
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
              });
          });



      $scope.go_to_problem = function(problem_id){
          $state.go('problem', {problem_id: problem_id});
      };

      $scope.leave_group = function(){
          var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
          $http.delete($scope.apiurl+'/group/leave/'+$stateParams.group_id, config)
            .success(function(data){
              $scope.group = data.group;
              $scope.member = false;
            })
            .error(function(error){
              var message = ((error && error.message) ? error.message : "Could not connect to server!");
                    console.log(error);
                    $scope.leader_alert = $alert({
                      content: message,
                      container: '.main-content',
                      placement: 'top',
                      type: 'error',
                      duration: 5
                    });
            });
      };

      $scope.join_group = function(){
          if($scope.group.public){
            var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
            $http.post($scope.apiurl+'/group/join/'+$stateParams.group_id, null, config)
              .success(function(data){
                $scope.group = data.group;
                $scope.get_me();
              })
              .error(function(error){
                var message = ((error && error.message) ? error.message : "Could not connect to server!");
                    console.log(error);
                    $scope.leader_alert = $alert({
                      content: message,
                      container: '.main-content',
                      placement: 'top',
                      type: 'error',
                      duration: 5
                    });
            });
          }
      };
});
