angular.module('cmb')
.controller('LeaderProbStatCtrl', function($scope, $http, $alert, $auth, $state, $stateParams, myConfig, _) {
    'use strict';
    $scope._ = _;
    $scope.apiurl = myConfig.apiurl+'/api';
    $scope.orderVar = '+';
    $scope.highscoreOrder = 'time';

    $scope.initialize = function(){
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.get($scope.apiurl+'/leader/'+$stateParams.group_id, config)
            .success(function(data) {
                if(data.message === "Leader"){
                    $scope.getProblem();
                } else {
                    $scope.leader_alert = $alert({
                        content: "Not leader. Redirected to home!",
                        container: '.main-content',
                        placement: 'top',
                        type: 'error',
                        duration: 5
                    });
                    $state.go('home');
                }
            })
            .error(function(error) {
                var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                console.log(error);
                $scope.leader_alert = $alert({
                  content: message,
                  container: '.main-content',
                  placement: 'top',
                  type: 'error',
                  duration: 5
                });
            });
    };

    $scope.getProblem = function(){
        $http.get($scope.apiurl+'/problem/'+$stateParams.problem_id)
                .success(function(data) {
                    $scope.problem = data.problem;
                    $scope.getGroup();
            })
            .error(function(error) {
                $scope.get_problem_alert = $alert({
                    content: 'Could not fetch problems from server',
                    container: '.main-content',
                    placement: 'top',
                    type: 'error',
                    duration: 5
                });
                console.log(error);
            });
    };

    $scope.getGroup = function(){
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.get($scope.apiurl+'/group/'+$stateParams.group_id, config)
            .success(function(data) {
                $scope.group = data.group;
                var list = _.filter($scope.problem.submissions, {'visible': true});
                _.forEach($scope.group.members, function(member){
                    var tmp = _.filter(list, {'user_id': member.user_id});
                    if(_.isEmpty(tmp)){
                        member.submission = {'state': 'Not Delivered'};
                        member.submission.stateStyle = {'color': 'red'};
                    } else {
                        member.submission = {'state': 'Submitted'};
                        member.submission.stateStyle = {'color': 'orange'};
                        _.forEach(tmp, function(sub){
                            if(sub.state === 'finished'){
                                member.submission = {'state': 'Success'};
                                member.submission.stateStyle = {'color': 'green'};
                            }
                        });
                    }
                });
                var hs = _.filter($scope.problem.submissions, {'state': 'finished', 'visible': true});
                hs = _.sortBy(hs, $scope.highscoreOrder);
                var best = [];
                var memberIDs = [];
                _.forEach($scope.group.members, function(member){
                    memberIDs.push(member.user_id);
                });
                $scope.problem.highscore = _.filter(hs, function(sub){
                    if(!_.includes(best, sub.user_id) && _.includes(memberIDs, sub.user_id)){
                        best.push(sub.user_id);
                        return sub;
                    }
                });
            })
            .error(function(error) {
                var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                console.log(error);
                $scope.leader_alert = $alert({
                  content: message,
                  container: '.main-content',
                  placement: 'top',
                  type: 'error',
                  duration: 5
                });
            });
    };

    $scope.sortHighscore = function(sort_by){
        $scope.highscoreOrder = sort_by;
        $scope.problem.submissions = _.filter($scope.problem.submissions, {'state': 'finished'});
        $scope.problem.submissions = _.sortBy($scope.problem.submissions, $scope.highscoreOrder);
        var best = [];
        $scope.problem.highscore = _.filter($scope.problem.submissions, function(sub){
            if(!_.includes(best, sub.user_id)){
                best.push(sub.user_id);
                return sub;
            }
        });
    };

    $scope.goodnessEnabled = function(){
        if(_.isUndefined($scope.problem)){
            return false;
        }
        return $scope.problem.goodness !== null;
    };

    $scope.filenameFormatter = function(filename){
        return filename.substring(0,filename.length-4);
    };


    $scope.go_to_user = function(user_id){
        $state.go('leader_user_stat', {'group_id': $stateParams.group_id, 'user_id': user_id});
    };

    $scope.go_to_group = function(){
            $state.go('leader', {group_id: $stateParams.group_id});
        };

    $scope.initialize();

});
