angular.module('cmb')
.controller('ProblemCtrl', function($scope, $sce, $http, $alert, $auth, $modal, $state,
                                    $stateParams, socketio, myConfig, _) {
  'use strict';
    $scope._ = _;
    $scope.apiurl = myConfig.apiurl+'/api';
    $scope.uploadFileButton = false;
    $scope.orderVar = '+';
    $scope.highscoreOrder = 'time';
    $scope.selectedHighscore = 'Public';
    $scope.dropdown = [{'text': 'Public', 'click':'changeHighscore(0)'}];
    $scope.allowedFileEndings = ['zip', 'cpp', 'c', 'h', 'hpp'];
    $scope.connected = false;
    $scope.server_timeout = 90;


    $scope.getProblem = function (){
        $http.get($scope.apiurl+'/problem/'+$stateParams.problem_id)
        .success(function(data) {
            if(data.message){
                $scope.get_problem_alert = $alert({
                content: data.message,
                type: 'material',
                duration: 5
                });
                $state.go('home');
            }
            $scope.problem = data.problem;
            $scope.problem.description = $sce.trustAsHtml($scope.problem.description);
            var hs = $scope.getHighscoreList($scope.problem.submissions);
            hs = _.sortBy(hs, $scope.highscoreOrder);
            var best = [];
            $scope.problem.highscore = _.filter(hs, function(sub){
                if(!_.includes(best, sub.user_id)){
                    best.push(sub.user_id);
                    return sub;
                }
            });
            if($scope.lastpageParams){
                if(!isNaN(parseInt($scope.lastpageParams.group_id))){
                    $scope.changeHighscore(parseInt($scope.lastpageParams.group_id));
                }
            }
        })
        .error(function(error) {
            $scope.get_problem_alert = $alert({
                content: 'Could not fetch problems from server',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
            console.log(error);
        });
    };

    $scope.getHighscoreList = function(sublist) {
      return _.filter(sublist, function(sub){
          if(!$scope.isNotFinished(sub) && sub.visible == true){
              return sub;
          }
      });
    };

    $scope.setGroupHighScore = function() {
      var hs = $scope.getHighscoreList($scope.problem.submissions);
      hs = _.sortBy(hs, $scope.highscoreOrder);
      var best = [];
      var memberIDs = [];
      _.forEach($scope.group.members, function(member){
          memberIDs.push(member.user_id);
      });
      $scope.problem.highscore = _.filter(hs, function(sub){
          if(!_.includes(best, sub.user_id) && _.includes(memberIDs, sub.user_id)){
              best.push(sub.user_id);
              return sub;
          }
      });
    };

    $scope.changeHighscore = function(group_id){
        if(group_id === 0){
            $scope.selectedHighscore = "Public";
            var hs = $scope.getHighscoreList($scope.problem.submissions);
            hs = _.sortBy(hs, $scope.highscoreOrder);
            var best = [];
            $scope.problem.highscore = _.filter(hs, function(sub){
                if(!_.includes(best, sub.user_id)){
                    best.push(sub.user_id);
                    return sub;
                }
            });
        } else {
            var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
            $http.get($scope.apiurl+'/group/'+group_id, config)
                .success(function(data) {
                    $scope.group = data.group;
                    $scope.selectedHighscore = $scope.group.name;
                    $scope.setGroupHighScore();
                })
                .error(function(error) {
                    var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                    console.log(error);
                    $scope.leader_alert = $alert({
                      content: message,
                      container: '.main-content',
                      placement: 'top',
                      type: 'error',
                      duration: 5
                    });
                });
        }
    };

    $scope.isAuthenticated = function() {
        return $auth.isAuthenticated();
    };

    $scope.refresh = function(){
        $scope.current_alert = $alert({
                content: 'Refreshing! Result will be returned if finished, this may take several minutes.',
                container: '.main-content',
                placement: 'top',
                type: 'info',
                duration: 5
                });
        $scope.getSubmissions();
        $scope.getProblem();
    };

    $scope.toggleVisible = function(sub_id){
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.put($scope.apiurl+'/submission/'+sub_id, null, config)
        .success(function() {
            $scope.getSubmissions();
            $scope.getProblem();
        })
        .error(function(error) {
            var message = ( error && error.message ? error.message : "Could not connect to server!");
            $scope.toggle_visible_alert = $alert({
                content: message,
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
            console.log(error);
        });
    };

    $scope.orderSubmissions = function(){
        if(!_.isEmpty($scope.submissions)){
            _.forEach($scope.submissions, function(sub){
                sub.best = false;
                if(sub.state == "in queue" || sub.state == "running"  ){
                    sub.energy = null;
                    sub.edp = null;
                    sub.time = null;
                }
            });
            $scope.submissions = _.filter($scope.submissions, {'problem_id':parseInt($stateParams.problem_id)});
            var tmp = $scope.getHighscoreList($scope.submissions);
            if(!_.isEmpty(tmp)){
                tmp = _.sortBy(tmp, $scope.highscoreOrder);
                var best_id = _.first(tmp).id;
                _.forEach($scope.submissions, function(sub){
                    if(sub.id == best_id){
                        sub.best = true;
                    }
                });
            }
            if($scope.group) {
              $scope.setGroupHighScore();
            }
        }
    };

    $scope.getSubmissions = function(){
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $scope.uploadFileButton = false;
        $http.get($scope.apiurl+'/submissions/'+$stateParams.problem_id, config)
        .success(function(data) {
            $scope.submissions = data.submissions;
            $scope.orderSubmissions();
            $scope.uploadFileButton = true;
        })
        .error(function(error) {
            $scope.get_subs_alert = $alert({
                content: 'Could not fetch submissions from server',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
            console.log(error);
        });
    };

    $scope.deleteSubmission = function(sub) {
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.delete($scope.apiurl+'/submission/'+sub.id, config)
          .success(function(data) {
            var message = "Submission " + sub.name + " was deleted.";
            $scope.submissions = _.filter($scope.submissions, function(s) { return s.id != sub.id; });
            $scope.delete_submission = $alert({
              content: message,
              container: '.main-content',
              placement: 'top',
              type: 'info',
              duration: 3
            });
          })
        .error(function(error) {
          var message = ( (error && error.message) ? error.message : "Could not connect to server!");
          console.log(error);
          $scope.delete_submission = $alert({
            content: message,
            container: '.main-content',
            placement: 'top',
            type: 'error',
            duration: 5
          });
        });
    };

    $scope.runScript = function(problemName, sub_id) {
        $scope.runscript_alert = $alert({
                content: 'Running Program! This may take several minutes. Refresh to get result if finished!',
                container: '.main-content',
                placement: 'top',
                type: 'info',
                duration: 5
                });
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.post($scope.apiurl+'/enqueue/'+problemName+"/"+sub_id+"/"+"False", null, config)
            .success(function(data) {
                $scope.submissions = data.user.submissions;
                $scope.orderSubmissions();
                $scope.getProblem();
            })
            .error(function(error) {
                $scope.runscript_alert = $alert({
                content: 'Could not run program.',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
                console.log('Error: ' + error);
            });
    };

    $scope.profiling = function(problemName, sub_id) {
        $scope.runscript_alert = $alert({
                content: 'Profiling Program! This may take several minutes.',
                container: '.main-content',
                placement: 'top',
                type: 'info',
                duration: 5
                });
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.post($scope.apiurl+'/enqueue/'+problemName+"/"+sub_id+"/"+"True", null, config)
            .success(function(data) {
              $scope.submissions = data.user.submissions;
              $scope.orderSubmissions();
              $scope.getProblem();
            })
            .error(function(error) {
                $scope.runscript_alert = $alert({
                content: 'Could not profile program.',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
                console.log('Error: ' + error);
            });
    };

    $scope.compile = function(problemName,sub_id) {
        $scope.compile_alert = $alert({
                content: 'Compiling File',
                container: '.main-content',
                placement: 'top',
                type: 'info',
                duration: 3
                });
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.post($scope.apiurl+'/compile/'+problemName+'/'+sub_id, null, config)
            .success(function(data) {
                $scope.submissions = data.user.submissions;
                $scope.orderSubmissions();
            })
            .error(function(data) {
                $scope.compile_alert = $alert({
                content: 'Could not compile program.',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
                });
                console.log('Error: ' + data);
            });
        };

    $scope.compileAndRun = function(problemName, sub_id) {
      $scope.compile_alert = $alert({
              content: 'Compiling File',
              container: '.main-content',
              placement: 'top',
              type: 'info',
              duration: 3
              });
        _.map($scope.submissions, function(sub) {
          if(sub.id === sub_id) {
            sub.button_disabled = true;
          }
        });
        var config = {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}};
        $http.post($scope.apiurl+'/compile/'+problemName+'/'+sub_id, null, config)
            .then(function(response) {
              var data = response.data;
              $scope.submissions = data.user.submissions;
              $scope.orderSubmissions();
              var failed = false;
              _.map($scope.submissions, function(elem) {
                if(sub_id == elem.id && elem.state === 'failed') {
                  throw elem.detailed_state;
                }
              });
              return $http.post($scope.apiurl+'/enqueue/'+problemName+"/"+sub_id+"/"+"False", null, config);
            })
            .then(function(response) {
              var data = response.data;
              $scope.submissions = data.user.submissions;
              $scope.orderSubmissions();
              $scope.getProblem();
            })
            .catch(function(response) {
              var data = response.data;
              $scope.submissions = data.user.submissions;
              $scope.orderSubmissions();
              $scope.getProblem();
              $scope.runscript_alert = $alert({
                content: 'Could not run program.',
                container: '.main-content',
                placement: 'top',
                type: 'error',
                duration: 5
              });
            });
    };

    $scope.parseZipFile = function(file, filename) {
      var zipFile = new JSZip(file);

      // Check directory count
      var dirCount = 0;
      var dirName = filename.split('.zip')[0];
      _.forEach(zipFile.files, function(subFile, name) {
        if(typeof(subFile) !== 'undefined' && (subFile.dir || name.split('/').length > 2)) {
          // Special case for OSX users
          if(name.toUpperCase() === "__MACOSX/") {
            zipFile.remove(name);
          } else {
            dirCount += 1;
            dirName = name.substring(0, name.length-1);
          }
        }
      });

      if(dirCount === 0) {
        zipFile.folder(dirName);
        _.forEach(zipFile.files, function(subFile, name) {
          if(!subFile.dir) {
            var nameSplit = name.split("/");
            zipFile.file(dirName + "/" + nameSplit[nameSplit.length-1], subFile.asUint8Array());
            zipFile.remove(name);
          }
        });
      } else if(dirCount > 1) {
        $scope.upload_file_alert = $alert({
            content: 'Upload failed: Multiple directories or nested directories within the zip is not allowed! Check out the HowTo-page on how to upload solutions.',
            container: '.main-content',
            placement: 'top',
            type: 'error',
            duration: 5
            });
        return;
      }

      // Move valid files not in a directory otherwise remove the file.
      _.forEach(zipFile.files, function(subFile, name) {
        var nameSplit = name.split("/");
        if(!subFile.dir) {
          var outsideDir = nameSplit.length == 1;
          var allowedFile = $scope.allowedFileEndings.indexOf(name.split(".").slice(-1)[0]) > 0;
          if(outsideDir && allowedFile) {
            zipFile.file(dirName + "/" + name, subFile.asUint8Array());
            zipFile.remove(name);
          } else if(allowedFile) {
            return;
          } else {
            zipFile.remove(name);
          }
        }
      });

      console.log(zipFile);

      return zipFile.generate({type: "blob", mimeType: "application/zip", platform: "UNIX"});
    };

    $scope.parseAndSendZip = function(event, filename, problemId) {
      var file = $scope.parseZipFile(event.target.result, filename);

      if(typeof(file) === "undefined") {
        return;
      }
      $scope.sendZip(file, filename, problemId);
    };

    $scope.sendZip = function(zipFile, filename, problemId) {
      // Upload data
      var form = new FormData();
      form.append("file", zipFile);
      var xhr = new XMLHttpRequest();

      xhr.addEventListener('load', function(){
        $scope.upload_file_alert = $alert({
            content: 'Succesfully uploaded file.',
            container: '.main-content',
            placement: 'top',
            type: 'success',
            duration: 3
            });
        $scope.getSubmissions();
      });

      xhr.addEventListener('error', function(e) {
        console.log(e);
        $scope.upload_file_alert = $alert({
            content: 'Upload to server failed: Please check the size of the zip-folder, and notify the administrators if you are sure that your folder-structure corresponds to the folder-structure described at the HowTo-page.',
            container: '.main-content',
            placement: 'top',
            type: 'error',
            duration: 5
            });
        xhr.abort();
        return;
      });

      xhr.open("post", $scope.apiurl+"/upload/"+problemId+"/"+$scope.problem.name+"/"+filename, true);
      if(localStorage.satellizer_token){
          xhr.setRequestHeader('Authorization', 'Bearer '+localStorage.satellizer_token);
      }
      xhr.send(form);
    };

    $scope.sendCpp = function(event, filename, problemId) {
      // Creates a zip folder with the single cpp/c file and sends
      var zipFile = new JSZip();
      var basename = filename.substring(0, filename.lastIndexOf('.'));
      zipFile.file(basename + "/" + filename, event.target.result);
      var file = zipFile.generate({type: "blob", mimeType: "application/zip", platform: "UNIX"});

      $scope.sendZip(file, basename+".zip", problemId);
    };

    $scope.uploadFile = function(problemId,file) {
        var fileExtension = file.name.substring(file.name.lastIndexOf('.'));
        //var fileEnding = file.name.substring(file.name.length - 4);
        if(fileExtension != ".zip" && fileExtension != ".cpp" && fileExtension != ".c") {
              $scope.upload_file_alert = $alert({
                  content: 'The file must be a ".zip/.c/.cpp" file',
                  container: '.main-content',
                  placement: 'top',
                  type: 'error',
                  duration: 3
                  });
        } else {
            $scope.upload_file_alert = $alert({
                content: 'Uploading file...',
                container: '.main-content',
                placement: 'top',
                type: 'info',
                duration: 3
                });
            var tmp = _.filter($scope.submissions, function(sub){
                return sub.name.indexOf(file.name.substring(0, file.name.lastIndexOf('.'))) != -1;
            });

            var filename = file.name;
            if(tmp.length > 0){
		var reA = /[^a-zA-Z]/g;
                var reN = /[^0-9]/g;
                tmp.sort(function(a,b) {
                  var aA = a.name.replace(reA, "");
                  var bA = b.name.replace(reA, "");
                  if(aA === bA) {
                      var aN = parseInt(a.name.replace(reN, ""), 10);
                      var bN = parseInt(b.name.replace(reN, ""), 10);
                      return aN === bN ? 0 : aN < bN ? 1 : -1;
                  } else {
                      return aA < bA ? 1 : -1;
                  }
                });
                var version = parseInt(tmp[0].name.split(new RegExp("_v|.zip|.c|.cpp"))[1]) + 1;
		console.log(version);
                version = isNaN(version) ? 1 : version;
                filename = file.name.substring(0, file.name.lastIndexOf('.'))+"_v"+version+fileExtension;
            }
            var fileReader = new FileReader();

            fileReader.addEventListener('load', function(event) {
              if (fileExtension == ".zip") {
                var result = $scope.parseAndSendZip(event, filename, problemId);
              } else {
                var result = $scope.sendCpp(event, filename, problemId);
              }

              if(typeof(result) === "undefined") {
                fileReader.abort();
              }
            });

            fileReader.addEventListener('error', function(e) {
              console.log(e);
              $scope.upload_file_alert = $alert({
                  content: 'File upload failed. Check the HowTo-page on how to upload files, and contact the system administrators if you got problems uploading files to the system in the future.',
                  container: '.main-content',
                  placement: 'top',
                  type: 'error',
                  duration: 5
                  });
            });

            fileReader.readAsArrayBuffer(file);

        }
    };

    $scope.sortHighscore = function(sort_by){
        $scope.highscoreOrder = sort_by;
        $scope.problem.submissions = $scope.getHighscoreList($scope.problem.submissions);
        $scope.problem.submissions = _.sortBy($scope.problem.submissions, $scope.highscoreOrder);
        var best = [];
        $scope.problem.highscore = _.filter($scope.problem.submissions, function(sub){
            if(!_.includes(best, sub.user_id)){
                best.push(sub.user_id);
                return sub;
            }
        });
        $scope.orderSubmissions();
    };

    $scope.showErrorMsg = function(sub){
        var $modalScope = $scope.$new(true);
        $modalScope.sub = sub;
        $scope.error_modal = $modal({
            scope: $modalScope,
            animation: 'am-fade-and-scale',
            templateUrl: '/templates/modal.error.tpl.html',
            controller: 'ErrorMsgCtrl'
        });
    };

    $scope.goodnessEnabled = function(){
        if(_.isUndefined($scope.problem)){
            return false;
        }
        return $scope.problem.goodness !== null;
    };

    $scope.filenameFormatter = function(filename){
        return filename.substring(0,filename.length-4);
    };

    $scope.sortSubs = function(sort_by){
        $scope.orderVar = sort_by;
    };

    $scope.getFinishedSubmissions = function() {
        return _.filter($scope.submissions, function(sub){
            if(!$scope.isNotFinished(sub)){
                return sub;
            }
        });
    };

    $scope.getStateMessage = function(sub) {
      if(!sub.detailed_state) {
        var string = sub.state.toLowerCase();
        return string || "error";
      }
      return _.replace(sub.detailed_state, /_/g, " ").toLowerCase();
    };

    $scope.isUploaded = function(state){
        return state.toLowerCase() == "uploaded";
    };

    $scope.isFailed = function(state){
        return state.toLowerCase() == "failed";
    };

    $scope.isRerun = function(state) {
      return state.toLowerCase() == "rerun";
    };

    $scope.isCompiled = function(state){
      return state.toLowerCase() == 'compiled' || state.toLowerCase() == 'finished' || state.toLowerCase() =='rerun';
    };

    $scope.isInQueue = function(state) {
      return state.toLowerCase() == 'in queue';
    };

    $scope.isRunning = function(state){
      return state.toLowerCase() == "running";
    };

    $scope.isProfiling = function(state){
      return state.toLowerCase() == "profiling";
    };

    $scope.isNotFinished = function(sub) {
      return sub.state.toLowerCase() != "finished" && sub.state.toLowerCase() != "profiling";
    };

    $scope.hasProfilingData = function(sub) {
      return sub.has_profiling_data == true;
    };

    $scope.getMyGroups = function(){
        $http.get($scope.apiurl+'/mygroups', {headers: {'Authorization': 'Bearer '+localStorage.satellizer_token}})
            .success(function(data) {
                $scope.mygroups = data.groups;
                _.forEach(data.groups, function(group){
                    var ingroup = false;
                    _.forEach(group.problems, function(problem){
                        if(problem.problem_id === parseInt($stateParams.problem_id)){
                            ingroup = true;
                        }
                    });
                    if(ingroup){
                        $scope.dropdown.push({'text': group.name, 'click': 'changeHighscore('+group.id+')'});
                    }
                });
            })
            .error(function(error) {
                var message = ( (error && error.message) ? error.message : "Could not connect to server!");
                console.log(error);
                $scope.home_alert = $alert({
                  content: message,
                  container: '.main-content',
                  placement: 'top',
                  type: 'error',
                  duration: 5
                });
            });
    };

    socketio.on('submission-finished', function(data) {
      _.map($scope.submissions, function(elem) {
        if(elem.id == data.sub_id) {
          var message = "Submission " + elem.name + " finished executing.";
          $scope.finished_alert = $alert({
            content: message,
            container: '.main-content',
            placement: 'top',
            type: 'success',
            duration: 5
          });
          $scope.getSubmissions();
        }
      });
    $scope.getProblem();
    });

    socketio.on('profiling-finished', function(data) {
      _.map($scope.submissions, function(elem) {
        if(elem.id == data.sub_id) {
          var message = "Profiling of " + elem.name + " finished successfully.";
          $scope.finished_alert = $alert({
            content: message,
            container: '.main-content',
            placement: 'top',
            type: 'success',
            duration: 5
          });
          $scope.getSubmissions();
        }
      });
    $scope.getProblem();
    });

    socketio.on('profiling-failed', function(data) {
      _.map($scope.submissions, function(elem) {
        if(elem.id == data.sub_id) {
          var message = "Profiling of " + elem.name + " failed (not enough datapoints or timeout).";
          $scope.finished_alert = $alert({
            content: message,
            container: '.main-content',
            placement: 'top',
            type: 'error',
            duration: 5
          });
          $scope.getSubmissions();
        }
      });
    $scope.getProblem();
    });

    socketio.on('submission-uploaded', function(data) {
      $scope.getSubmissions();
    });

    socketio.on('submission-deleted', function(data) {
      $scope.getSubmissions();
    });

    // for debugging purposes
    socketio.on('message', function(data) {
      console.log(data.msg);
    });


    socketio.on('submission-dequeued', function(data) {
      if($scope.submissions && $scope.submissions.length > 0) {
        _.map($scope.submissions, function(elem) {
          if(elem.id === data.sub_id) {
            var message = "Submission " + elem.name + " started executing. Results should be ready within " +
                          data.running_time + " minutes.";
            $scope.started_running_alert = $alert({
              content: message,
              container: '.main-content',
              placement: 'top',
              type: 'info',
              duration: 6
            });
          }
        });
      }
      $scope.getSubmissions();
    });

    socketio.on('submission-enqueued', function(data) {
       _.map($scope.submissions, function(elem) {
         if(data.sub_id === elem.id) {
           var message = "Submitted " + elem.name + " as #" + data.queue_number + " in queue.";
           if(parseInt(data.queue_number) >= 3) {
             message += " Some waiting time is expected.";
           }
           $scope.enqueued_alert = $alert({
             content: message,
             container: '.main-content',
             placement: 'top',
             type: 'info',
             duration: 6
           });
         }
         /* If position in queue returned from server
         if($scope.submissions && $scope.submissions.length > 0) {
           var my_sub = false;
           _.map($scope.submissions, function(elem) {
             if(elem.id == data.sub_id) {
               my_sub = true;
               $scope.submissions_extras.push({
                 id: elem.id,
                 queue_number: parseInt(data.queue_number),
                 timer: -1
               });
             }
           });
           if(!my_sub) {
             _.map($scope.submissions_extras, function(elem) {
                 elem.queue_number = Math.max(elem.queue_number-1, 0);
             });
           }
         }
       */
       });
    });

    $scope.goodnessXorProfiling = function() {
	return ($scope.profilingEnabled && !$scope.goodnessEnabled()) || (!$scope.profilingEnabled() && $scope.goodnessEnabled());
    }

    $scope.profilingEnabled = function() {
        if(_.isUndefined($scope.problem)){
            return false;
        }
	return $scope.problem.allow_profiling == true;
    };

    $scope.$on('$stateChangeStart', function(event, toState, toParams, fromState, fromParams, options){
      socketio.emit('leave-problem', {'prob_id': $stateParams.problem_id});
      socketio.removeAllListeners();
    });

    $scope.$on('$destroy', function(event) {
      socketio.removeAllListeners();
    });

    $scope.$on('$stateChangeSuccess', function (ev, to, toParams, from, fromParams) {
        $scope.lastpage = from.name;
        $scope.lastpageParams = fromParams;
    });

    $scope.show_profiling_data = function(sub_id){
      $state.go('profiling', {submission_id: sub_id});
    };

    $scope.getProblem();
    socketio.forward('message');
    socketio.emit('join-problem', {'prob_id': $stateParams.problem_id});
    if($scope.isAuthenticated()){
        $scope.getMyGroups();
        $scope.getSubmissions();
    }
});
