describe('Unit: ErrorMsgCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    // Create the controller
    ctrl = $controller('ErrorMsgCtrl', {
      $scope: scope,
    });


  }));

  describe('Helpers: ', function(){

    it('isCompilationError:', function(){
      var test = scope.isCompilationError("SERVERSIDE_COMPILATION_FAILED");
      expect(test).toBeTruthy();
      var test1 = scope.isCompilationError(null);
      expect(test1).toBeFalsy();
      var test2 = scope.isCompilationError("SERVERSIDE_COMPILATION");
      expect(test2).toBeFalsy();
    });

    it('isRuntimeError:', function(){
      var test = scope.isRuntimeError("CORRECTNESS_RUNTIME_ERROR");
      expect(test).toBeTruthy();
      var test1 = scope.isRuntimeError(null);
      expect(test1).toBeFalsy();
      var test2 = scope.isRuntimeError("SERVERSIDE_RUNTIME");
      expect(test2).toBeFalsy();
    });

    it('isFalsyOutput:', function(){
      var test = scope.isFalsyOutput("CORRECTNESS_BAD_OUTPUT");
      expect(test).toBeTruthy();
      var test1 = scope.isFalsyOutput(null);
      expect(test1).toBeFalsy();
      var test2 = scope.isFalsyOutput("PROGRAM_BAD_OUTPUT");
      expect(test2).toBeFalsy();
    });

    it('isTimeoutError:', function(){
      var test = scope.isTimeoutError("SERVERSIDE_TIMEOUT");
      expect(test).toBeTruthy();
      var test1 = scope.isTimeoutError(null);
      expect(test1).toBeFalsy();
      var test2 = scope.isTimeoutError("SERVERSIDE_TIME");
      expect(test2).toBeFalsy();
    });

    it('isOtherError:', function(){
      var test = scope.isOtherError("SERVERSIDE_TIMEOUT");
      expect(test).toBeFalsy();
      var test1 = scope.isOtherError(null);
      expect(test1).toBeTruthy();
      var test2 = scope.isOtherError("SERVERSIDE_TIME");
      expect(test2).toBeFalsy();
    });

    it('parseRuntimeError:', function(){
      var msg = "Some super runtime error! Signal: SIGSEV Output: None";
      scope.parseRuntimeError(msg);
      expect(scope.splitMessage.length).toBe(3);
      expect(scope.splitMessage[0].trim()).toEqual("Some super runtime error!");
      expect(scope.splitMessage[1].trim()).toEqual("SIGSEV");
      expect(scope.splitMessage[2].trim()).toEqual("None");
    });

    it('parseFalsyOutput:', function(){
      var msg = "Some super false output! Result: SIGSEV was: None";
      scope.parseFalsyOutput(msg);
      expect(scope.splitMessage.length).toBe(3);
      expect(scope.splitMessage[0].trim()).toEqual("Some super false output!");
      expect(scope.splitMessage[1].trim()).toEqual("SIGSEV");
      expect(scope.splitMessage[2].trim()).toEqual("None");
    });

    it('parseCompilationError', function(){
      var msg = "Some super runtime error! Signal: SIGSEV Output: None";
      scope.parseCompilationError(msg);
      expect(scope.splitMessage.length).toBe(1);
      expect(scope.splitMessage[0].trim()).toEqual("Some super runtime error! Signal: SIGSEV Output: None");
    });

    it('parseTimeoutError', function(){
      var msg = "Some super runtime error! Signal: SIGSEV Output: None";
      scope.parseTimeoutError(msg);
      expect(scope.splitMessage.length).toBe(1);
      expect(scope.splitMessage[0].trim()).toEqual("Some super runtime error! Signal: SIGSEV Output: None");
    });

  });
});
