describe('Unit: GroupCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope, $stateParams, $auth) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    stateParams = $stateParams;
    stateParams.group_id = 1;
    // Create the controller
    ctrl = $controller('GroupCtrl', {
      $scope: scope,
      $stateParams: stateParams
    });


  }));

  it('go_to_problem(1) should route to problem 1',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_problem(1);
      expect($state.go).toHaveBeenCalledWith('problem', {problem_id: 1});
    }));

  it('isAuthenticated should call auth.isAuthenticated',
      inject(function($auth){
        spyOn($auth, 'isAuthenticated').and.callFake(function() {});
        scope.isAuthenticated();
        expect($auth.isAuthenticated).toHaveBeenCalled();
      }));

  describe('should request group automatically: ', function(){
    beforeEach(inject(function($httpBackend, $auth) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
      spyOn($auth, 'isAuthenticated').and.callFake(function(){
          return false;
      });
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('success and public', function(){
      var response = {'group': {'public': true}};
      httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
      httpBackend.flush();
      expect(scope.group).toEqual({'public': true});
      expect(scope.groupState).toEqual('Public');
    });

    it('success and private', function(){
      var response = {'group': {'public': false}};
      httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
      httpBackend.flush();
      expect(scope.group).toEqual({'public': false});
      expect(scope.groupState).toEqual('Private');
    });

    it('failed', function(){
      var response = {'submission': "submission"};
      httpBackend.expectGET(scope.apiurl+'/group/1').respond(500, response);
      httpBackend.flush();

      expect(scope.submission).toBeUndefined();
    });
  });

  describe('functions with http requests: ', function(){
    beforeEach(inject(function($httpBackend, $auth) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

  it('redirect_if_leader should redirect to leader page if leader',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      var response = {'message': "Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      scope.redirect_if_leader();
      httpBackend.flush();
      expect($state.go).toHaveBeenCalledWith('leader', {group_id: 1});
    }));

  it('redirect_if_leader should not redirect to leader page if not leader',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      var response = {'message': "Not Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      scope.redirect_if_leader();
      httpBackend.flush();
      expect($state.go).not.toHaveBeenCalled();
    }));

  it('redirect_if_leader should not redirect to leader page if not leader: failed',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      var response = {'message': "error"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(500, response);
      scope.redirect_if_leader();
      httpBackend.flush();
      expect($state.go).not.toHaveBeenCalled();
      expect(scope.group_alert.$options.content).toEqual('error');
    }));

  it('get_me should get states if i am a member: submitted',
    function(){
      scope.group = {'members': [{'user_id':1}], 'problems':[{'problem_id':1}]};
      var response = {'user': {'id':1, 'submissions': [{'problem_id':1}]}};
      httpBackend.expectGET(scope.apiurl+'/me').respond(200, response);
      scope.get_me();
      httpBackend.flush();
      expect(scope.group.problems[0].state).toEqual("Submitted");
    });

  it('get_me should get states if i am a member: Not delivered',
    function(){
      scope.group = {'members': [{'user_id':1}], 'problems':[{'problem_id':1}]};
      var response = {'user': {'id':1, 'submissions': [{'problem_id':2}]}};
      httpBackend.expectGET(scope.apiurl+'/me').respond(200, response);
      scope.get_me();
      httpBackend.flush();
      expect(scope.group.problems[0].state).toEqual("Not Delivered");
    });

  it('get_me should get states if i am a member: Success',
    function(){
      scope.group = {'members': [{'user_id':1}], 'problems':[{'problem_id':1}]};
      var response = {'user': {'id':1, 'submissions': [{'problem_id':1, 'state':"finished"}]}};
      httpBackend.expectGET(scope.apiurl+'/me').respond(200, response);
      scope.get_me();
      httpBackend.flush();
      expect(scope.group.problems[0].state).toEqual("Success");
    });

  it('leave_group should send leave_group request',
    function(){
      var response = {'group': 'group'};
      httpBackend.expectDELETE(scope.apiurl+'/group/leave/1').respond(200, response);
      scope.leave_group();
      httpBackend.flush();
      expect(scope.group).toEqual('group');
      expect(scope.member).toBeFalsy();
    });

  it('leave_group should send leave_group request: failed',
    function(){
      scope.member = true;
      scope.group = {'public':true};
      var response = {'group': 'group'};
      httpBackend.expectDELETE(scope.apiurl+'/group/leave/1').respond(500, response);
      scope.leave_group();
      httpBackend.flush();
      expect(scope.group).toEqual({'public':true});
      expect(scope.member).toBeTruthy();
    });

  it('join_group should send join_group request',
    function(){
      spyOn(scope, 'get_me').and.callFake(function() {});
      scope.group = {'public':true};
      var response = {'group': 'group'};
      httpBackend.expectPOST(scope.apiurl+'/group/join/1').respond(200, response);
      scope.join_group();
      httpBackend.flush();
      expect(scope.group).toEqual('group');
      expect(scope.get_me).toHaveBeenCalled();
    });

  it('join_group should send join_group request: failed',
    function(){
      spyOn(scope, 'get_me').and.callFake(function() {});
      scope.group = {'public':true};
      var response = {'group': 'group'};
      httpBackend.expectPOST(scope.apiurl+'/group/join/1').respond(500, response);
      scope.join_group();
      httpBackend.flush();
      expect(scope.group).toEqual({'public':true});
      expect(scope.get_me).not.toHaveBeenCalled();
    });
  });


});
