describe('Unit: HomeCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope, $auth) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    spyOn($auth, 'isAuthenticated').and.callFake(function() {});
    // Create the controller
    ctrl = $controller('HomeCtrl', {
      $scope: scope
    });
  }));

  it('go_to_problem(1) should route to problem 1 page', 
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_problem(1);
      expect($state.go).toHaveBeenCalledWith('problem', {problem_id: 1});
    }));

  it('go_to_group(1,"member") should route to group 1 page', 
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_group(1,"member");
      expect($state.go).toHaveBeenCalledWith('group', {group_id: 1});
    }));

  it('go_to_group(1,"leader") should route to leader 1 page', 
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_group(1, "leader");
      expect($state.go).toHaveBeenCalledWith('leader', {group_id: 1});
    }));

  it('isAuthenticated should call $auth.isAuthenticated', 
    inject(function($auth){
      scope.isAuthenticated();
      expect($auth.isAuthenticated).toHaveBeenCalled();
    }));

  it('should get problems automatically: success', 
    inject(function($httpBackend, $auth) {     
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');

        var response = {'problems': ["problem"]};
        httpBackend.expectGET(scope.apiurl+'/problems').respond(200, response);
        httpBackend.flush();
        expect(scope.problems).toEqual(["problem"]);
        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
        expect($auth.isAuthenticated).toHaveBeenCalled();
    }));

  it('should get problems automatically: failure', 
    inject(function($httpBackend) {      
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');

        var response = {'message': "error"};
        httpBackend.expectGET(scope.apiurl+'/problems').respond(500, response);
        httpBackend.flush();
        expect(scope.problems).toBeUndefined();

        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
    }));

  it('get_groups should call api/mygroups and update scope: success', 
    inject(function($httpBackend) {      
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');

        scope.get_groups();
        var response = {'groups': ["group"]};
        httpBackend.expectGET(scope.apiurl+'/mygroups').respond(200, response);
        httpBackend.flush();
        expect(scope.mygroups).toEqual(["group"]);

        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
    }));

  it('get_groups should call api/mygroups and update scope: failure', 
    inject(function($httpBackend) {      
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');

        scope.get_groups();
        var response = {'message': 'error'};
        httpBackend.expectGET(scope.apiurl+'/mygroups').respond(500, response);
        httpBackend.flush();
        expect(scope.mygroups).toBeUndefined();
        expect(scope.home_alert.$options.content).toEqual('error');

        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
    }));
});
