describe('Unit: LeaderCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope, $stateParams) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    scope.groupUrlParam = '?with-submissions=1';

    stateParams = $stateParams;
    stateParams.group_id = 1;
    // Create the controller
    ctrl = $controller('LeaderCtrl', {
      $scope: scope,
      $stateParams: stateParams
    });

  }));

  it('go_to_problem(1) should route to problem 1',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_problem(1);
      expect($state.go).toHaveBeenCalledWith('problem', {problem_id: 1});
    }));

  it('should initialize automatically', function(){
    expect(scope.init).toBeTruthy();
  });

  describe('when initializing: ', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('should check with server if leader', function(){
      response = {'message': 'Leader'};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      httpBackend.flush();
    });

    describe('if leader: ', function(){
      beforeEach(inject(function($httpBackend) {
        response = {'message': 'Leader'};
        httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      }));

      it('should get problems: success', function(){
        var response = {'problems': ["problem"]};
        httpBackend.expectGET(scope.apiurl+'/problems').respond(200, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.problems).toEqual(["problem"]);
      });

      it('should get problems: failure', function(){
        var response = {'message': "error"};
        httpBackend.expectGET(scope.apiurl+'/problems').respond(500, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.problems).toBeUndefined();
      });

      it('should get users: success', function(){
        var response = {'users': ["user"]};
        httpBackend.expectGET(scope.apiurl+'/users').respond(200, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.users).toEqual(["user"]);
      });

      it('should get users: failure', function(){
        var response = {'message': "error"};
        httpBackend.expectGET(scope.apiurl+'/users').respond(500, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.users).toBeUndefined();
      });

      it('should get group data: success', function(){
        var response = {'group': "group"};
        httpBackend.expectGET(scope.apiurl+'/group/1?with-submissions=1').respond(200, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.group).toEqual('group');
      });

      it('should get group data: failure', function(){
        var response = {'message': "error"};
        httpBackend.expectGET(scope.apiurl+'/group/1?with-submissions=1').respond(500, response);

        scope.initialize();
        httpBackend.flush();
        expect(scope.group).toBeUndefined();
      });
    });
  });

  describe('add/delete requests: ', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function(){
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('remove_problem(1) should ask server to delete problem 1: success', function(){
      scope.group = {'id':1};
      var response = {'group': "group"};
      httpBackend.expectDELETE(scope.apiurl+'/group/1/problem/1', function(headers){
        return !_.isUndefined(headers.Authorization);
      }).respond(200, response);

      scope.remove_problem(1);
      httpBackend.flush();
      expect(scope.group).toEqual("group");
    });

    it('remove_problem(1) should ask server to delete problem 1: failure', function(){
      scope.group = {'id':1};
      var response = {'message': "error"};
      httpBackend.expectDELETE(scope.apiurl+'/group/1/problem/1').respond(500, response);

      scope.remove_problem(1);
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
      expect(scope.leader_alert.$options.content).toEqual('error');
    });

    it('remove_member(1) should ask server to remove user 1 from group: success', inject(function($state){
      scope.group = {'id':1};
      var response = {'group': "group"};
      httpBackend.expectDELETE(scope.apiurl+'/group/1/member/1', function(headers){
        return !_.isUndefined(headers.Authorization);
      }).respond(200, response);

      scope.remove_member(1);
      httpBackend.flush();
      expect(scope.group).toEqual("group");
    }));

    it('remove_member(1) should ask server to remove user 1 from group: failure', function(){
      scope.group = {'id':1};
      var response = {'message': "error"};
      httpBackend.expectDELETE(scope.apiurl+'/group/1/member/1').respond(500, response);

      scope.remove_member(1);
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
      expect(scope.leader_alert.$options.content).toEqual('error');
    });

    it('add_member(1) should not do anything if no member is selected', inject(function($state){
      scope.group = {'id':1};
      scope.add_member();
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
    }));

    it('add_member() should ask server to add user 1 to group: success', inject(function($state){
      scope.group = {'id':1};
      scope.selectedUser = {'id': 1};
      var response = {'group': "group"};
      httpBackend.expectPOST(scope.apiurl+'/group/1/member/1', null, function(headers){
        return !_.isUndefined(headers.Authorization);
      }).respond(200, response);
      scope.add_member();
      httpBackend.flush();
      expect(scope.group).toEqual("group");
    }));

    it('add_member() should ask server to add user 1 to group: failure', inject(function($state){
      scope.group = {'id':1};
      scope.selectedUser = {'id': 1};
      var response = {'message': "error"};
      httpBackend.expectPOST(scope.apiurl+'/group/1/member/1').respond(500, response);
      scope.add_member();
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
      expect(scope.leader_alert.$options.content).toEqual('error');
    }));

     it('add_problem() should not do anything if no problem is selected', inject(function($state){
      scope.group = {'id':1};
      scope.add_problem();
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
    }));

    it('add_problem() should ask server to add problem 1 to group: success', inject(function($state){
      scope.group = {'id':1};
      scope.selectedProblem = {'id': 1};
      var response = {'group': "group"};
      httpBackend.expectPOST(scope.apiurl+'/group/1/problem/1', null, function(headers){
        return !_.isUndefined(headers.Authorization);
      }).respond(200, response);
      scope.add_problem();
      httpBackend.flush();
      expect(scope.group).toEqual("group");
    }));

    it('add_member(1) should ask server to add problem 1 to group: failure', inject(function($state){
      scope.group = {'id':1};
      scope.selectedProblem = {'id': 1};
      var response = {'message': "error"};
      httpBackend.expectPOST(scope.apiurl+'/group/1/problem/1').respond(500, response);
      scope.add_problem();
      httpBackend.flush();
      expect(scope.group).toEqual({'id':1});
      expect(scope.leader_alert.$options.content).toEqual('error');
    }));
  });

});
