describe('Unit: LeaderProbStatsCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope, $stateParams) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    stateParams = $stateParams;
    stateParams.group_id = 1;
    stateParams.problem_id = 1;
    // Create the controller
    ctrl = $controller('LeaderProbStatCtrl', {
      $scope: scope,
      $stateParams: stateParams
    });
    spyOn(scope, 'initialize').and.callFake(function() {});


  }));

  it('go_to_group() should route to group page',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_group();
      expect($state.go).toHaveBeenCalledWith('leader', {group_id: 1});
    }));

  it('go_to_user(1) should route to user page',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_user(1);
      expect($state.go).toHaveBeenCalledWith('leader_user_stat', {group_id: 1, user_id: 1});
    }));

  it('filenameFormatter should remove last four letters from filename',
      function() {
        var filename = "filename.zip";
        expect(filename).toEqual("filename.zip");
        filename = scope.filenameFormatter(filename);
        expect(filename).toEqual("filename");
      });

  it('goodnessEnabled should return true if problem has goodness defined',
    function(){
        expect(scope.problem).toBeUndefined();
        expect(scope.goodnessEnabled()).toEqual(false);
        scope.problem = {'goodness':null};
        expect(scope.problem).toBeDefined();
        expect(scope.goodnessEnabled()).toEqual(false);
        scope.problem.goodness = "something";
        expect(scope.goodnessEnabled()).toEqual(true);
    });

  describe('initialize should: ', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('check if leader', function(){
      var response = {'message': "Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      scope.initialize();
      httpBackend.flush();
    });

    it('check if leader failed', function(){
      var response = {'message': "error"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(500, response);
      scope.initialize();
      httpBackend.flush();
      expect(scope.leader_alert.$options.content).toEqual('error');
    });

    it('if not leader, redirect to home',
      inject(function($state){
        spyOn($state, 'go').and.callFake(function(state) {});
        var response = {'message': "Not Leader"};
        httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
        scope.initialize();
        httpBackend.flush();
        expect($state.go).toHaveBeenCalledWith('home');
      }));

    it('if leader, call getProblem', function(){
      spyOn(scope, 'getProblem').and.callFake(function(){});
      var response = {'message': "Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);

      scope.initialize();
      httpBackend.flush();
      expect(scope.getProblem).toHaveBeenCalled();
    });
  });

  describe('get', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('Problem should request problem 1',
      function(){
        spyOn(scope, 'getGroup').and.callFake(function(){});
        var response = {'problem': 'problem'};
        httpBackend.expectGET(scope.apiurl+'/problem/1').respond(200, response);
        scope.getProblem();
        httpBackend.flush();
        expect(scope.problem).toEqual('problem');
        expect(scope.getGroup).toHaveBeenCalled();
      });

    it('Group should request group 1, delivered',
      function(){
        scope.problem = {'submissions': [{'id':1,'state':'finished', 'problem_id':1,'user_id':2, 'edp':20, 'energy':5, 'time':4, 'goodness':1, 'visible':true}]};
        var response = {'group': {'members':[{'user_id': 2}]}};
        httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
        scope.getGroup();
        httpBackend.flush();
        expect(scope.group.members[0].submission.stateStyle).toEqual({'color': 'green'});
      });

    it('Group should request group 1, not delivered',
      function(){
        scope.problem = {'submissions': []};
        var response = {'group': {'members':[{'user_id': 1}]}};
        httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
        scope.getGroup();
        httpBackend.flush();
        expect(scope.group.members[0].submission.stateStyle).toEqual({'color': 'red'});
      });
  });

  describe('sortHighscore should sort on ',
    function(){
      beforeEach(function() {
        scope.problem = {};
        sub1 = {'state':'finished', 'user_id':1, 'edp':10, 'energy':5, 'time':4, 'goodness':1};
        sub2 = {'state':'finished', 'user_id':2, 'edp':11, 'energy':4, 'time':1, 'goodness':4};
        sub3 = {'state':'finished', 'user_id':3, 'edp':12, 'energy':2, 'time':2, 'goodness':3};
        sub4 = {'state':'finished', 'user_id':1, 'edp':13, 'energy':3, 'time':3, 'goodness':2};
        scope.problem.submissions = [sub1,sub2,sub3,sub4];
      });

      it('edp',
        function(){
          scope.sortHighscore('edp');
          expect(scope.problem.highscore).toEqual([sub1,sub2,sub3]);
        });

      it('energy',
        function(){
          scope.sortHighscore('energy');
          expect(scope.problem.highscore).toEqual([sub3,sub4,sub2]);
          });

      it('time',
        function(){
          scope.sortHighscore('time');
          expect(scope.problem.highscore).toEqual([sub2,sub3,sub4]);
          });

      it('goodness',
        function(){
          scope.sortHighscore('goodness');
          expect(scope.problem.highscore).toEqual([sub1,sub3,sub2]);
          });
    });

});
