describe('Unit: LeaderUserStatsCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope, $stateParams) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    stateParams = $stateParams;
    stateParams.group_id = 1;
    stateParams.user_id = 1;
    // Create the controller
    ctrl = $controller('LeaderUserStatCtrl', {
      $scope: scope,
      $stateParams: stateParams
    });
    spyOn(scope, 'initialize').and.callFake(function() {});
  }));

  it('go_to_group() should route to group page',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_group();
      expect($state.go).toHaveBeenCalledWith('leader', {group_id: 1});
    }));

  it('go_to_problem_stat(1) should route to group page',
    inject(function($state){
      spyOn($state, 'go').and.callFake(function(state, params) {});
      scope.go_to_problem_stat(1);
      expect($state.go).toHaveBeenCalledWith('leader_prob_stat', {group_id: 1, problem_id: 1});
    }));

  describe('initialize should: ', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('check if leader', function(){
      var response = {'message': "Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      scope.initialize();
      httpBackend.flush();
    });

    it('check if leader failed', function(){
      var response = {'message': "error"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(500, response);
      scope.initialize();
      httpBackend.flush();
      expect(scope.leader_alert.$options.content).toEqual('error');
    });

    it('if not leader, redirect to home',
      inject(function($state){
        spyOn($state, 'go').and.callFake(function(state) {});
        var response = {'message': "Not Leader"};
        httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
        scope.initialize();
        httpBackend.flush();
        expect($state.go).toHaveBeenCalledWith('home');
      }));

    it('if leader, call getUser', function(){
      spyOn(scope, 'getUser').and.callFake(function(){});
      var response = {'message': "Leader"};
      httpBackend.expectGET(scope.apiurl+'/leader/1').respond(200, response);
      scope.initialize();
      httpBackend.flush();
      expect(scope.getUser).toHaveBeenCalled();
    });
  });

  describe('get', function(){
    beforeEach(inject(function($httpBackend) {
      httpBackend = $httpBackend;
      httpBackend.whenGET(/^.*$/).respond(500, '');
    }));

    afterEach(function() {
      httpBackend.verifyNoOutstandingExpectation();
      httpBackend.verifyNoOutstandingRequest();
    });

    it('User should request user 1',
      function(){
        spyOn(scope, 'getGroup').and.callFake(function(){});
        var response = {'user': 'user'};
        httpBackend.expectGET(scope.apiurl+'/user/1').respond(200, response);
        scope.getUser();
        httpBackend.flush();
        expect(scope.user).toEqual('user');
        expect(scope.getGroup).toHaveBeenCalled();
      });

    it('Group should request group 1, delivered',
      function(){
        scope.user = {'submissions': [{'id':1,'state':'finished', 'problem_id':1,'user_id':2, 'edp':20, 'energy':5, 'time':4, 'goodness':1, 'visible':true}]};
        var response = {'group': {'problems':[{'problem_id': 1}]}};
        httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
        scope.getGroup();
        httpBackend.flush();
        expect(scope.group.problems[0].submission.stateStyle).toEqual({'color': 'green'});
      });

    it('Group should request group 1, not delivered',
      function(){
        scope.user = {'submissions': []};
        var response = {'group': {'problems':[{'problem_id': 1}]}};
        httpBackend.expectGET(scope.apiurl+'/group/1').respond(200, response);
        scope.getGroup();
        httpBackend.flush();
        expect(scope.group.problems[0].submission.stateStyle).toEqual({'color': 'red'});
      });
  });

});
