describe('Unit LoginCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;

  beforeEach(inject(function($httpBackend,  $rootScope, $auth, $state, $q) {      
    httpBackend = $httpBackend;
    httpBackend.whenGET(/^.*$/).respond(500, '');
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    spyOn($state, 'go').and.callFake(function(state, params) {});
  }));

  afterEach(function() {
    httpBackend.flush();
    httpBackend.verifyNoOutstandingExpectation();
    httpBackend.verifyNoOutstandingRequest();
  });

  it('should login and redirect to last page', inject(function($controller, $auth, $state, $q) {
    spyOn($auth, 'login').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    // Create the controller
    ctrl = $controller('LoginCtrl', {
      $scope: scope
    });
    scope.name = 'test';
    scope.password = 'pw';
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'home'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.login();    
    scope.$digest();
    expect($auth.login).toHaveBeenCalledWith({name: 'test', password: btoa('pw')});
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should login and redirect to home when coming from signup', inject(function($controller, $auth, $state, $q) {
    spyOn($auth, 'login').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    // Create the controller
    ctrl = $controller('LoginCtrl', {
      $scope: scope
    });
    scope.name = 'test';
    scope.password = 'pw';
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'signup'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.login();    
    scope.$digest();
    expect($auth.login).toHaveBeenCalledWith({name: 'test', password: btoa('pw')});
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should login and redirect to home when coming from newpw', inject(function($controller, $auth, $state, $q) {
    spyOn($auth, 'login').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    // Create the controller
    ctrl = $controller('LoginCtrl', {
      $scope: scope
    });
    scope.name = 'test';
    scope.password = 'pw';
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'newpw'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.login();    
    scope.$digest();
    expect($auth.login).toHaveBeenCalledWith({name: 'test', password: btoa('pw')});
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should create alert if login promise rejected', inject(function($controller, $auth, $state, $q) {
    // Create the controller
    spyOn($auth, 'login').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.reject('Remote call result');
      return deferred.promise;
    });

    ctrl = $controller('LoginCtrl', {
      $scope: scope
    });
    scope.name = 'test';
    scope.password = 'pw';
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'home'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.login();    
    scope.$digest();
    expect($auth.login).toHaveBeenCalledWith({name: 'test', password: btoa('pw')});
    expect(scope.login_alert.$options.content).toEqual("Could not connect to server!");
  }));
});