describe('Unit LogoutCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  beforeEach(inject(function($httpBackend) {      
    httpBackend = $httpBackend;
    httpBackend.whenGET(/^.*$/).respond(500, '');
  }));
  afterEach(function() {
    httpBackend.flush();
    httpBackend.verifyNoOutstandingExpectation();
    httpBackend.verifyNoOutstandingRequest();
  });

    it('should logout if user logged in, and redirect to last page', inject(function($controller, $rootScope, $auth, $state, $q) {
    var ctrl, scope;
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    spyOn($auth, 'isAuthenticated').and.callFake(function() {
      return true;
    });
    spyOn($auth, 'logout').and.callFake(function(){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    spyOn($state, 'go').and.callFake(function(state, params) {});

    // Create the controller
    ctrl = $controller('LogoutCtrl', {
      $scope: scope
    });

    scope.$broadcast('$stateChangeSuccess', null, null, {name:'home'}, null);
    scope.$apply();
    expect(scope.lastpage).toEqual('home');
    expect($auth.isAuthenticated).toHaveBeenCalled();
    expect($auth.logout).toHaveBeenCalled();
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should logout if user logged in, and redirect to home if last page is profile', inject(function($controller, $rootScope, $auth, $state, $q) {
    var ctrl, scope;
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    spyOn($auth, 'isAuthenticated').and.callFake(function() {
      return true;
    });
    spyOn($auth, 'logout').and.callFake(function(){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    spyOn($state, 'go').and.callFake(function(state, params) {});

    // Create the controller
    ctrl = $controller('LogoutCtrl', {
      $scope: scope
    });

    scope.$broadcast('$stateChangeSuccess', null, null, {name:'profile'}, null);
    scope.$apply();
    expect(scope.lastpage).toEqual('home');
    expect(scope.logout_alert.$options.content).toEqual('You have been logged out');
    expect($auth.isAuthenticated).toHaveBeenCalled();
    expect($auth.logout).toHaveBeenCalled();
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should not logout if user not logged in', inject(function($controller, $rootScope, $auth, $state, $q) {
    var ctrl, scope;
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();

    spyOn($auth, 'isAuthenticated').and.callFake(function() {
      return false;
    });
    spyOn($state, 'go').and.callFake(function(state, params) {});

    // Create the controller
    ctrl = $controller('LogoutCtrl', {
      $scope: scope
    });
    expect($auth.isAuthenticated).toHaveBeenCalled();
    expect($state.go).toHaveBeenCalledWith('home');
  }));
});