describe('Unit NewGroupCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($rootScope) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
  }));

  describe('isAuthenticated true: ', function(){
      beforeEach(inject(function($controller, $auth, $httpBackend, $state) {
        spyOn($state, 'go').and.callFake(function(state, params) {});
        spyOn($auth, 'isAuthenticated').and.callFake(function() {
          return true;
        });
        ctrl = $controller('NewGroupCtrl', {
          $scope: scope
        });
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');
      }));

      afterEach(function() {
        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
      });

      it('create_group should create a group if successful', inject(function($state){
        scope.name = "simen";
        scope.description = "er best";
        var response = {'group': {'id': 1}};
        httpBackend.expectPOST(scope.apiurl+'/group/create').respond(200, response);

        scope.create_group();
        httpBackend.flush();
        expect($state.go).toHaveBeenCalledWith('leader', {'group_id': 1});
      }));

      it('create_group should alert if name empty', inject(function($state){
        scope.name = "";
        scope.description = "er best";

        scope.create_group();
        expect(scope.create_group_alert.$options.content).toEqual('Name must be filled out.');
        httpBackend.flush();
      }));

      it('create_group should alert if unsuccessful', inject(function($state){
        spyOn(console, 'log').and.callFake(function(){});
        scope.name = "simen";
        scope.description = "er best";
        httpBackend.expectPOST(scope.apiurl+'/group/create').respond(500);

        scope.create_group();
        httpBackend.flush();
        expect(scope.create_group_alert.$options.content).toEqual('Could not create group.');
        expect(console.log).toHaveBeenCalled();
      }));
  });

  describe('isAuthenticated false: ', function(){
      beforeEach(inject(function($auth, $controller, $state) {
        spyOn($state, 'go').and.callFake(function(state) {});
        spyOn($auth, 'isAuthenticated').and.callFake(function() {
          return false;
        });
        ctrl = $controller('NewGroupCtrl', {
          $scope: scope
        });
      }));

      it('should route to login', inject(function($auth, $state){
        expect($auth.isAuthenticated).toHaveBeenCalled();
        expect($state.go).toHaveBeenCalledWith('login');
      }));

  });

});
