describe('ProblemCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;
  // inject the $controller and $rootScope services
  // in the beforeEach block
  beforeEach(inject(function($controller, $rootScope) {
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    // Create the controller
    ctrl = $controller('ProblemCtrl', {
      $scope: scope
    });
  }));

  it('showErrorMsg should display modal',
    inject(function($modal) {
      scope.showErrorMsg({id: 1, msg: "test"});
      expect(scope.error_modal.$options.scope.sub).toEqual({id: 1, msg: "test"});
    }));

  it('refresh method',
    inject(function($alert){
      spyOn(scope, 'getSubmissions').and.callFake(function() {});
      spyOn(scope, 'getProblem').and.callFake(function() {});

      scope.refresh();
      expect(scope.current_alert.$options.content).toEqual('Refreshing! Result will be returned if finished, this may take several minutes.');
      expect(scope.getSubmissions).toHaveBeenCalled();
      expect(scope.getProblem).toHaveBeenCalled();
    }));

  describe('Functions with http calls: ',
    function(){
      beforeEach(inject(function($httpBackend) {
        httpBackend = $httpBackend;
        httpBackend.whenGET(/^.*$/).respond(500, '');
      }));

      afterEach(function() {
        httpBackend.verifyNoOutstandingExpectation();
        httpBackend.verifyNoOutstandingRequest();
      });

      describe('compile http post request: ', function() {
        it('success',
        function(){
          var response = JSON.stringify({'user':{'submissions':['sub']}});
          spyOn(scope, 'orderSubmissions').and.callFake(function() {});
          httpBackend.expectPOST(scope.apiurl+'/compile/problem_name/'+1, undefined, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);
          scope.compile("problem_name", 1);
          httpBackend.flush();
          expect(scope.compile_alert.$options.content).toEqual('Compiling File');
          expect(scope.orderSubmissions).toHaveBeenCalled();
          expect(scope.submissions).toEqual(['sub']);
        });

        it('failed',
          function(){
            var response = JSON.stringify('error msg');
            httpBackend.expect('POST', scope.apiurl+'/compile/problem_name/'+1)
                .respond(500, response);
            scope.compile("problem_name", 1);
            httpBackend.flush();
            expect(scope.compile_alert.$options.content).toEqual('Could not compile program.');
        });
      });

      describe('toggleVisible http put request: ', function() {
        it('success',
        function(){
          spyOn(scope, 'getSubmissions').and.callFake(function() {});
          spyOn(scope, 'getProblem').and.callFake(function() {});
          httpBackend.expectPUT(scope.apiurl+'/submission/'+1, undefined, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200);
          scope.toggleVisible(1);
          httpBackend.flush();
          expect(scope.getSubmissions).toHaveBeenCalled();
          expect(scope.getProblem).toHaveBeenCalled();
        });

        it('failed',
          function(){
            var response = {'message':'error msg'};
            httpBackend.expect('PUT', scope.apiurl+'/submission/'+1)
                .respond(500, response);
            scope.toggleVisible(1);
            httpBackend.flush();
            expect(scope.toggle_visible_alert.$options.content).toEqual('error msg');
        });
      });

      describe('runScript http post request: ', function() {
        it('success', function(){
          var response = JSON.stringify({'user':{'submissions':['sub']}});
          spyOn(scope, 'orderSubmissions').and.callFake(function() {});
          spyOn(scope, 'getProblem').and.callFake(function(){});
          httpBackend.expectPOST(scope.apiurl+'/enqueue/problem_name/'+1+'/'+'False', undefined, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);
          scope.runScript("problem_name", 1);
          httpBackend.flush();
          expect(scope.runscript_alert.$options.content).toEqual('Running Program! This may take several minutes. Refresh to get result if finished!');
          expect(scope.orderSubmissions).toHaveBeenCalled();
          expect(scope.getProblem).toHaveBeenCalled();
          expect(scope.submissions).toEqual(['sub']);
        });

        it('failed',
          function(){
            var response = JSON.stringify('error msg');
            httpBackend.expect('POST', scope.apiurl+'/enqueue/problem_name/'+1+'/'+'False')
                .respond(500, response);
            scope.runScript("problem_name", 1);
            httpBackend.flush();

        });
      });

      describe('compileAndRun http post request: ', function() {
        it('success', function(){
          var response = JSON.stringify({'user':{'submissions':['sub']}});
          spyOn(scope, 'orderSubmissions').and.callFake(function() {});
          spyOn(scope, 'getProblem').and.callFake(function(){});
          httpBackend.expectPOST(scope.apiurl+'/compile/problem_name/'+1)
              .respond(200, response);
          httpBackend.expectPOST(scope.apiurl+'/enqueue/problem_name/'+1+'/'+'False')
              .respond(200, response);
          scope.compileAndRun("problem_name", 1);
          httpBackend.flush();
          expect(scope.compile_alert.$options.content).toEqual('Compiling File');
          expect(scope.orderSubmissions).toHaveBeenCalled();
          expect(scope.getProblem).toHaveBeenCalled();
          expect(scope.submissions).toEqual(['sub']);
        });

        it('failed',
          function(){
            spyOn(scope, 'orderSubmissions').and.callFake(function() {});
            spyOn(scope, 'getProblem').and.callFake(function(){});
            var response = JSON.stringify({'user':{'submissions':['sub']}});
            httpBackend.expectPOST(scope.apiurl+'/compile/problem_name/'+1)
                .respond(500, response);
            scope.compileAndRun("problem_name", 1);
            httpBackend.flush();
            expect(scope.compile_alert.$options.content).toEqual('Compiling File');
            expect(scope.orderSubmissions).toHaveBeenCalled();
            expect(scope.getProblem).toHaveBeenCalled();
            expect(scope.runscript_alert.$options.content).toEqual('Could not run program.');
        });
      });

      describe('getSubmissions http get request: ', function() {
        it('success', inject(function($stateParams){
          $stateParams.problem_id = 1;
          var response = JSON.stringify({'submissions':['sub']});
          spyOn(scope, 'orderSubmissions').and.callFake(function() {});
          httpBackend.expectGET(scope.apiurl+'/submissions/'+1, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);
          scope.getSubmissions();
          httpBackend.flush();
          expect(scope.orderSubmissions).toHaveBeenCalled();
          expect(scope.submissions).toEqual(['sub']);
          expect(scope.uploadFileButton).toBeTruthy();
        }));

        it('failed', inject(function($stateParams){
          $stateParams.problem_id = 1;
          var response = JSON.stringify('error msg');
          httpBackend.expectGET(scope.apiurl+'/submissions/'+1)
              .respond(500, response);
          scope.getSubmissions();
          httpBackend.flush();
          expect(scope.get_subs_alert.$options.content).toEqual('Could not fetch submissions from server');
        }));
      });

      describe('deleteSubmission http delete request: ', function() {
        it('success', inject(function($stateParams){
          $stateParams.problem_id = 1;
          var response = JSON.stringify({'message': 'Submission 1 deleted.'});
          scope.submissions = [{id: 1, name: "test"}];
          httpBackend.expectDELETE(scope.apiurl+'/submission/'+1, function(headers){
            return !_.isUndefined(headers.Authorization);
          }).respond(200, response);
          scope.deleteSubmission({id: 1, name: "test"});
          httpBackend.flush();
          expect(scope.submissions).toEqual([]);
          expect(scope.delete_submission.$options.content).toEqual('Submission test was deleted.');
        }));

        it('failed', inject(function($stateParams){
          $stateParams.problem_id = 1;
          var response = JSON.stringify('error msg');
          httpBackend.expectDELETE(scope.apiurl+'/submission/'+1)
              .respond(500, "");
          scope.deleteSubmission({id: 1, name: "test"});
          httpBackend.flush();
          expect(scope.delete_submission.$options.content).toEqual('Could not connect to server!');
        }));
      });

      describe('getProblem http get request: ', function() {
        it('success', inject(function($stateParams){
          $stateParams.problem_id = 1;
          sub1 = {'id':1,'state':'finished', 'problem_id':1,'user_id':2, 'edp':20, 'energy':5, 'time':4, 'goodness':1, 'visible':true};
          sub2 = {'id':2,'state':'running', 'problem_id':1,'user_id':1, 'edp':4, 'energy':4, 'time':1, 'goodness':4, 'visible':true};
          sub3 = {'id':3,'state':'finished', 'problem_id':1,'user_id':1, 'edp':4, 'energy':2, 'time':2, 'goodness':3, 'visible':true};
          sub4 = {'id':4,'state':'finished', 'problem_id':1,'user_id':2, 'edp':15, 'energy':3, 'time':5, 'goodness':2, 'visible':true};
          sub5 = {'id':3,'state':'finished', 'problem_id':1,'user_id':1, 'edp':2, 'energy':2, 'time':1, 'goodness':3, 'visible':false};

          var response = JSON.stringify({'problem':{'submissions': [sub1,sub2,sub3,sub4]}});

          hs1 = {'id':1,'state':'finished', 'problem_id':1,'user_id':2, 'edp':20, 'energy':5, 'time':4, 'goodness':1, 'visible':true};
          hs3 = {'id':3,'state':'finished', 'problem_id':1,'user_id':1, 'edp':4, 'energy':2, 'time':2, 'goodness':3, 'visible':true};
          httpBackend.expectGET(scope.apiurl+'/problem/'+1).respond(200, response);
          scope.getProblem();
          httpBackend.flush();
          expect(scope.problem.highscore).toEqual([hs3,hs1]);
        }));

        it('failed', inject(function($stateParams){
          $stateParams.problem_id = 1;
          var response = JSON.stringify('error msg');
          httpBackend.expectGET(scope.apiurl+'/problem/'+1)
              .respond(500, response);
          scope.getProblem();
          httpBackend.flush();
          expect(scope.get_problem_alert.$options.content).toEqual('Could not fetch problems from server');
        }));
      });

      describe('uploadFile filereader: ', function() {

        it('success with new filename', function(){
          scope.problem = {};
          scope.problem.name = "problem_name";
          var file = new Blob(["filename.zip"], { type: 'text/html' });
          file.name = "filename.zip";
          spyOn(FileReader.prototype, 'readAsArrayBuffer').and.callThrough();
          spyOn(FileReader.prototype, 'addEventListener');
          spyOn(scope, 'sendZip').and.callFake(function(f) {});
          scope.uploadFile(1, file);
          expect(scope.upload_file_alert.$options.content).toEqual('Uploading file...');
          expect(FileReader.prototype.readAsArrayBuffer).toHaveBeenCalledWith(file);
          expect(FileReader.prototype.addEventListener).toHaveBeenCalled();
          // If needed
          //expect(scope.sendZip).toHaveBeenCalled();
          httpBackend.flush();
        });

        it('success with existing filename', function(){
          scope.problem = {};
          scope.problem.name = "problem_name";
          scope.submissions = [{'name':"filename.zip"}];
          var file = {};
          file.name = "filename.zip";
          spyOn(FileReader.prototype, 'readAsArrayBuffer').and.callFake(function(f) {});
          scope.uploadFile(1, file);
          httpBackend.flush();
          expect(scope.upload_file_alert.$options.content).toEqual('Uploading file...');
          expect(FileReader.prototype.readAsArrayBuffer).toHaveBeenCalledWith(file);
        });
      });

      describe('parseZipFile', function() {

        it('should display alert with multiple directories', function(){
          var zipFileObj = {
            files: {
                "test1/": {
                  dir: true
                },
                "test2/": {
                  dir: true
                }
            },
          };
          spyOn(window, 'JSZip').and.returnValue(zipFileObj);
          scope.parseZipFile({}, "test");

          expect(scope.upload_file_alert.$options.content).toEqual('Upload failed: Multiple directories or nested directories within the zip is not allowed! Check out the HowTo-page on how to upload solutions.');
          httpBackend.flush();
        });

        it('should remove __MACOSX folder', function(){
          var zipFileObj = {
            files: {
                "__MACOSX/": {
                  dir: true
                }
            },
            generate: jasmine.createSpy(),
            remove: jasmine.createSpy(),
            folder: jasmine.createSpy()
          };
          spyOn(window, 'JSZip').and.returnValue(zipFileObj);
          scope.parseZipFile({}, "test");
          expect(zipFileObj.folder).toHaveBeenCalled();
          expect(zipFileObj.remove).toHaveBeenCalled();
          expect(zipFileObj.generate).toHaveBeenCalled();
          httpBackend.flush();
        });

        it('should move create directory if do not exist', function(){
          var zipFileObj = {
            files: {
                "test.c": {
                  dir: false,
                  asUint8Array: jasmine.createSpy()
                }
            },
            generate: jasmine.createSpy(),
            remove: jasmine.createSpy(),
            folder: jasmine.createSpy(),
            file: jasmine.createSpy()
          };
          spyOn(window, 'JSZip').and.returnValue(zipFileObj);
          scope.parseZipFile({}, "test");
          expect(zipFileObj.generate).toHaveBeenCalled();
          expect(zipFileObj.remove).toHaveBeenCalled();
          expect(zipFileObj.file).toHaveBeenCalled();

          httpBackend.flush();
        });

        it('should move files not in a directory', function(){
          var zipFileObj = {
            files: {
                "test.c": {
                  dir: false,
                  asUint8Array: jasmine.createSpy()
                },
                "test/": {
                  dir: true,
                  asUint8Array: jasmine.createSpy()
                },
                "test/test1.c": {
                  dir: false,
                  asUint8Array: jasmine.createSpy()
                },
                "test.pdf": {
                  dir: false,
                  asUint8Array: jasmine.createSpy()
                }
            },
            generate: jasmine.createSpy(),
            remove: jasmine.createSpy(),
            folder: jasmine.createSpy(),
            file: jasmine.createSpy()
          };
          spyOn(window, 'JSZip').and.returnValue(zipFileObj);
          scope.parseZipFile({}, "test");
          expect(zipFileObj.generate).toHaveBeenCalled();
          expect(zipFileObj.remove).toHaveBeenCalled();
          expect(zipFileObj.file).toHaveBeenCalled();
          httpBackend.flush();
        });

      });

      describe('sendZip http post request: ', function() {

        it('should remove unncassery files', function(){
          // If possible, check that everything is setup
          xhrObj = jasmine.createSpyObj('xhrObj',
                                      ['addEventListener', 'open', 'send', 'setRequestHeader']);

          spyOn(window, "XMLHttpRequest").and.returnValue(xhrObj);

          scope.problem = {};
          scope.problem.name = "problem_name";
          var file = new Blob(["filename.zip"], { type: 'text/html' });
          file.name = "filename.zip";
          spyOn(scope, 'parseZipFile').and.callFake(function(f) {});
          scope.sendZip({target: {result: file}},"filename.zip",1);
          httpBackend.flush();
          // If needed
          //expect(scope.sendZip).toHaveBeenCalled();
        });

      });

    describe('changeHighscore should change to ',
      function(){
        beforeEach(function() {
          scope.problem = {};
          sub1 = {'state':'finished', 'user_id':1, 'edp':10, 'energy':5, 'time':4, 'goodness':1, 'visible':true};
          sub2 = {'state':'finished', 'user_id':2, 'edp':11, 'energy':4, 'time':1, 'goodness':4, 'visible':true};
          sub3 = {'state':'finished', 'user_id':3, 'edp':12, 'energy':2, 'time':2, 'goodness':3, 'visible':true};
          sub4 = {'state':'finished', 'user_id':1, 'edp':13, 'energy':3, 'time':3, 'goodness':2, 'visible':true};
          scope.problem.submissions = [sub1,sub2,sub3,sub4];
        });

        it('public if group id is 0',
          function(){
            scope.changeHighscore(0);
            expect(scope.selectedHighscore).toEqual("Public");
            expect(scope.problem.highscore).toEqual([sub2,sub3,sub4]);
            httpBackend.flush();
          });

        it('request group with id 1 when group id is 1',
          function(){
            response = {'group':{'members': [{'user_id': 1},{'user_id': 2}], 'name':'group_name'}};
            httpBackend.expectGET(scope.apiurl+'/group/'+1).respond(200, response);

            scope.changeHighscore(1);
            httpBackend.flush();
            expect(scope.selectedHighscore).toEqual('group_name');
            expect(scope.problem.highscore).toEqual([sub2,sub4]);
          });

        it('alert user when error', function(){
          response = {'message': 'error'};
          httpBackend.expectGET(scope.apiurl+'/group/'+1).respond(500, response);
          scope.changeHighscore(1);
          httpBackend.flush();
          expect(scope.leader_alert.$options.content).toEqual('error');
        });
      });

      describe('getMyGroups should ',
        function(){
          it('request mygroups',
            inject(function($stateParams){
              $stateParams.problem_id = 1;
              response = {'groups':[{'id':1, 'name': 'gruppenavn' ,'problems': [{'problem_id': 1}]}]};
              httpBackend.expectGET(scope.apiurl+'/mygroups').respond(200, function(headers){
                return !_.isUndefined(headers.Authorization);
              }).respond(200, response);
              scope.getMyGroups();
              httpBackend.flush();
              expect(scope.dropdown[0].text).toEqual('Public');
              expect(scope.dropdown[1].text).toEqual('gruppenavn');
              expect(scope.mygroups).toEqual([{'id':1, 'name': 'gruppenavn' ,'problems': [{'problem_id': 1}]}]);
            }));
        });
    });

  describe('orderSubmissions should ',
    function(){
      beforeEach(function() {
        spyOn(scope, 'setGroupHighScore').and.callThrough();
        sub1 = {'id':1,'state':'finished', 'problem_id':1,'user_id':1, 'edp':20, 'energy':5, 'time':4, 'goodness':1, 'visible': true};
        sub2 = {'id':2,'state':'running', 'problem_id':1,'user_id':1, 'edp':null, 'energy':4, 'time':1, 'goodness':4, 'visible': true};
        sub3 = {'id':3,'state':'Uploaded', 'problem_id':2,'user_id':1, 'edp':null, 'energy':2, 'time':2, 'goodness':3,
        'visible': true};
        sub4 = {'id':4,'state':'finished', 'problem_id':1,'user_id':1, 'edp':15, 'energy':3, 'time':5, 'goodness':2,
        'visible': true};
        scope.submissions = [sub1,sub2,sub3,sub4];
      });

      it('handle empty list',
        function(){
          scope.submissions = [];
          scope.orderSubmissions();
          expect(scope.submissions).toEqual([]);
          expect(scope.setGroupHighScore).not.toHaveBeenCalled();
        });

      it('have edp,time,energy only for finished submissions',
        function(){
          scope.orderSubmissions();
          expect(sub1.edp).toEqual(20);
          expect(sub2.edp).toEqual(null);
          expect(sub3.edp).toEqual(null);
          expect(sub4.edp).toEqual(15);

          expect(sub1.energy).toEqual(5);
          expect(sub1.time).toEqual(4);
          expect(sub2.energy).toEqual(null);
          expect(sub2.time).toEqual(null);

          expect(scope.setGroupHighScore).not.toHaveBeenCalled();
        });

      it('set "best" to true if sub in highscore',
        inject(function($stateParams){
          $stateParams.problem_id = 1;

          scope.highscoreOrder = 'edp';
          scope.orderSubmissions();
          expect(sub1.best).toBeFalsy();
          expect(sub2.best).toBeFalsy();
          expect(sub3.best).toBeFalsy();
          expect(sub4.best).toBeTruthy();

          scope.highscoreOrder = 'energy';
          scope.orderSubmissions();
          expect(sub1.best).toBeFalsy();
          expect(sub2.best).toBeFalsy();
          expect(sub3.best).toBeFalsy();
          expect(sub4.best).toBeTruthy();

          scope.highscoreOrder = 'time';
          scope.orderSubmissions();
          expect(sub1.best).toBeTruthy();
          expect(sub2.best).toBeFalsy();
          expect(sub3.best).toBeFalsy();
          expect(sub4.best).toBeFalsy();

          scope.highscoreOrder = 'goodness';
          scope.orderSubmissions();
          expect(sub1.best).toBeTruthy();
          expect(sub2.best).toBeFalsy();
          expect(sub3.best).toBeFalsy();
          expect(sub4.best).toBeFalsy();

          expect(scope.setGroupHighScore).not.toHaveBeenCalled();
        }));

      it('sort out non-group submissions',
        inject(function($stateParams) {
          $stateParams.problem_id = 1;
          scope.problem = {};
          scope.problem.submissions = scope.submissions;
          scope.group = {'members': [{'user_id': 1}] ,'problems': [{'problem_id': 1}]};
          scope.submissions.push({'id':5,'state':'finished', 'problem_id':1,'user_id':2, 'edp':15, 'energy':3, 'time':5, 'goodness':2, 'visible': true});

          scope.highscoreOrder = 'edp';
          scope.orderSubmissions();

          expect(scope.setGroupHighScore).toHaveBeenCalled();
          expect(scope.problem.highscore).toEqual([sub4]);
        }));
    });

  describe('sortHighscore should ',
    function(){
      beforeEach(function() {
        spyOn(scope, 'orderSubmissions').and.callFake(function() {});
        scope.problem = {};
        sub1 = {'state':'finished', 'user_id':1, 'edp':10, 'energy':5, 'time':4, 'goodness':1, 'visible': true};
        sub2 = {'state':'finished', 'user_id':2, 'edp':11, 'energy':4, 'time':1, 'goodness':4, 'visible': true};
        sub3 = {'state':'finished', 'user_id':3, 'edp':12, 'energy':2, 'time':2, 'goodness':3, 'visible': true};
        sub4 = {'state':'finished', 'user_id':1, 'edp':13, 'energy':3, 'time':3, 'goodness':2, 'visible': true};
        sub5 = {'state':'finished', 'user_id':5, 'edp':13, 'energy':3, 'time':3, 'goodness':2, 'visible': false};
        scope.problem.submissions = [sub1,sub2,sub3,sub4,sub5];
      });

      it(' sort on edp',
        function(){
          scope.sortHighscore('edp');
          expect(scope.problem.highscore).toEqual([sub1,sub2,sub3]);
          expect(scope.orderSubmissions).toHaveBeenCalled();
        });

      it(' sort on energy',
        function(){
          scope.sortHighscore('energy');
          expect(scope.problem.highscore).toEqual([sub3,sub4,sub2]);
          expect(scope.orderSubmissions).toHaveBeenCalled();
          });

      it(' sort on time',
        function(){
          scope.sortHighscore('time');
          expect(scope.problem.highscore).toEqual([sub2,sub3,sub4]);
          expect(scope.orderSubmissions).toHaveBeenCalled();
          });

      it(' sort on goodness',
        function(){
          scope.sortHighscore('goodness');
          expect(scope.problem.highscore).toEqual([sub1,sub3,sub2]);
          expect(scope.orderSubmissions).toHaveBeenCalled();
          });

      it(' filter non-visible entries',
        function(){
          scope.sortHighscore('time');
          expect(scope.problem.highscore).toEqual([sub2,sub3,sub4]);
          expect(scope.orderSubmissions).toHaveBeenCalled();

          expect(scope.problem.highscore[0].visible).toBe(true);
          expect(scope.problem.highscore[1].visible).toBe(true);
          expect(scope.problem.highscore[2].visible).toBe(true);
      });
    });

  describe('Helper functions:', function(){
    it('isAuthenticated should call auth.isAuthenticated',
      inject(function($auth){
        spyOn($auth, 'isAuthenticated').and.callFake(function() {});
        scope.isAuthenticated();
        expect($auth.isAuthenticated).toHaveBeenCalled();
      }));

    it('isFailed should only return true with state "failed"',
      function() {
        expect(scope.isFailed('failed')).toEqual(true);
        expect(scope.isFailed('running')).toEqual(false);
      });

    it('isUploaded should only return true with state "Uploaded"',
      function() {
        expect(scope.isUploaded('Uploaded')).toEqual(true);
        expect(scope.isUploaded('Compiled')).toEqual(false);
      });

    it('isCompiled should only return true with state "Compiled" or "finished"',
      function() {
        expect(scope.isCompiled('Compiled')).toEqual(true);
        expect(scope.isCompiled('finished')).toEqual(true);
        expect(scope.isCompiled('Uploaded')).toEqual(false);
      });

    it('isRunning should only return true with state "running"',
      function() {
        expect(scope.isRunning('in queue')).toEqual(false);
        expect(scope.isRunning('running')).toEqual(true);
        expect(scope.isRunning('Uploaded')).toEqual(false);
      });

    it('isInQueue should only return true with state "in queue"',
       function() {
         expect(scope.isInQueue('in queue')).toEqual(true);
         expect(scope.isInQueue('running')).toEqual(false);
         expect(scope.isInQueue('Uploaded')).toEqual(false);
       });

     it('isRerun should only return true with state "rerun"',
        function() {
          expect(scope.isRerun('rerun')).toEqual(true);
          expect(scope.isRerun('running')).toEqual(false);
          expect(scope.isRerun('Uploaded')).toEqual(false);
        });

     it('isNotFinished should only return false with state "finished"',
        function() {
          expect(scope.isNotFinished({state: 'in queue'})).toEqual(true);
          expect(scope.isNotFinished({state: 'running'})).toEqual(true);
          expect(scope.isNotFinished({state: 'finished'})).toEqual(false);
        });

    it('sortSubs should alter orderVar',
      function() {
        scope.orderVar = 'edp';
        expect(scope.orderVar).toEqual('edp');
        scope.sortSubs('energy');
        expect(scope.orderVar).toEqual('energy');
      });

    it('getStateMessage should return error string according to state',
      function() {
        expect(scope.getStateMessage({state: "error message"})).toEqual("error message");
        expect(scope.getStateMessage({state: ""})).toEqual("error");
        expect(scope.getStateMessage({state: "error message", detailed_state: "COMPILATION_ERROR"})).toEqual("compilation error");
      });

    it('filenameFormatter should remove last four letters from filename',
      function() {
        var filename = "filename.zip";
        expect(filename).toEqual("filename.zip");
        filename = scope.filenameFormatter(filename);
        expect(filename).toEqual("filename");
      });

    it('goodnessEnabled should return true if problem has goodness defined',
      function(){
          expect(scope.problem).toBeUndefined();
          expect(scope.goodnessEnabled()).toEqual(false);
          scope.problem = {'goodness':null};
          expect(scope.problem).toBeDefined();
          expect(scope.goodnessEnabled()).toEqual(false);
          scope.problem.goodness = "something";
          expect(scope.goodnessEnabled()).toEqual(true);
      });
  });

});
