describe('Unit SignupCtrl: ', function() {
  // Load the module with Controller
  beforeEach(module('cmb'));

  var ctrl, scope;

  beforeEach(inject(function($httpBackend,  $rootScope, $auth, $state, $q) {      
    httpBackend = $httpBackend;
    httpBackend.whenGET(/^.*$/).respond(500, '');
    // Create a new scope that's a child of the $rootScope
    scope = $rootScope.$new();
    scope.email = 'email';
    scope.name = 'test';
    scope.password = 'pw';
    spyOn($state, 'go').and.callFake(function(state, params) {});
  }));

  afterEach(function() {
    httpBackend.flush();
    httpBackend.verifyNoOutstandingExpectation();
    httpBackend.verifyNoOutstandingRequest();
  });

  it('should signup and redirect to last page', inject(function($controller, $auth, $state, $q) {
    scope.confirm_password = "pw";
    scope.confirm_email = "email";
    spyOn($auth, 'signup').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.resolve('Remote call result');
      return deferred.promise;
    });
    // Create the controller
    ctrl = $controller('SignupCtrl', {
      $scope: scope
    });
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'home'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.signup();    
    scope.$digest();
    expect($auth.signup).toHaveBeenCalledWith({email:'email', name: 'test', password: btoa('pw')});
    expect($state.go).toHaveBeenCalledWith('home', null);
  }));

  it('should create alert if signup promise rejected', inject(function($controller, $auth, $state, $q) {
    scope.confirm_password = "pw";
    scope.confirm_email = "email";
    spyOn($auth, 'signup').and.callFake(function(data){
      var deferred = $q.defer();
      deferred.reject('Remote call result');
      return deferred.promise;
    });
    // Create the controller
    ctrl = $controller('SignupCtrl', {
      $scope: scope
    });
    scope.$broadcast('$stateChangeSuccess', null, null, {name:'home'}, null);
    scope.$apply();

    expect(scope.lastpage).toEqual('home');
    scope.signup();    
    scope.$digest();
    expect($auth.signup).toHaveBeenCalledWith({email:'email', name: 'test', password: btoa('pw')});
    expect(scope.signup_alert.$options.content).toEqual("Could not connect to server!");
  }));

  it('should check if password and confirm password is equal', inject(function($controller){
    scope.confirm_password = "not_pw";
    ctrl = $controller('SignupCtrl', {
      $scope: scope
    });
    scope.signup();
    expect(scope.signup_alert.$options.content).toEqual("Passwords did not match!");
    expect(scope.password).toEqual("");
    expect(scope.confirm_password).toEqual("");
  }));

  it('should check if password and confirm password is equal', inject(function($controller){
    scope.confirm_password = "pw";
    scope.confirm_email = "not_email";
    ctrl = $controller('SignupCtrl', {
      $scope: scope
    });

    scope.signup();
    expect(scope.signup_alert.$options.content).toEqual("Emails did not match!");
    expect(scope.email).toEqual("");
    expect(scope.confirm_email).toEqual("");
  }));
});