#!/bin/bash
#set -e

DIR="$HOME/cmb-board/mountBlanc"
WS="$DIR/files/workspace"
. $DIR/backend.cfg

trap 'kill $PID' TERM INT EXIT

function cleanup
{
	chmod o-rx $WS
	rm -rf $WS
	if [ -f $DIR/perf.txt ]; then
		rm $DIR/perf.txt
	fi
	exit
}

function main
{
	# Give all users write/execute access to workspace
	chmod o+rx $WS
	cd $WS
	
	# Make program
	make -Bs
	
    # Run program on main input
	set +e
	./program < input.txt > /dev/null &
	PID=$!
	bash $DIR/poormansprofiler.sh $PID > $WS/prof.txt &
	perf stat -o $DIR/perf.out -x: -e cycles,instructions,L1-dcache-loads,L1-dcache-load-misses,LLC-loads,LLC-load-misses -p $PID
	set -e
	
	if ps -p $PID; then
		kill $PID
	fi
	
	cd $DIR

	# Create json of perf data
	json=`python perf2json.py perf.out`

	# Create flame graph
	if [ -f flame_graph.svg ]; then
		rm flame_graph.svg
	fi
	python stackcollapse-gdb.py $WS/prof.txt > $WS/stacks.txt
	set +e
	cat $WS/stacks.txt | ~/FlameGraph/flamegraph.pl --color=java --hash > flame_graph.svg
	if [[ $? != 0 ]] ; then
		echo "Failed to create flame_graph"
		if [ -f flame_graph.svg ]; then
			rm flame_graph.svg
		fi
	fi

	echo $json 1>&3
	cleanup
}

# Stdout and stderr are redirected to /dev/null if no argument is passed (= NO DEBUG)
exec 3>&1
exec 4>&2
if [ $# -gt 0 ]
then
	DEBUG="True"
else
	DEBUG="False"
	exec 1>/dev/null
	exec 2>/dev/null
fi

main
