from datetime import datetime
from functools import wraps

from flask import g, jsonify, request
from jwt import DecodeError, ExpiredSignature

from helpers import parse_token


class MyWrappers():
    def init_app(self, app):
        self.app = app

    def login_required(self, f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if not request.headers.get('Authorization'):
                response = jsonify(message='Missing authorization header')
                response.status_code = 401
                return response

            payload = parse_token(self.app, request.headers.get('Authorization').split()[1])

            if datetime.fromtimestamp(payload['exp']) < datetime.now():
                response = jsonify(message='Token has expired')
                response.status_code = 401
                return response

            g.user_id = payload['sub']

            return f(*args, **kwargs)

        return decorated_function

    def fetch_login_status(self, f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if not request.headers.get('Authorization'):
                g.user_id = None
            else:
                try:
                    payload = parse_token(self.app, request.headers.get('Authorization').split()[1])
                    g.user_id = payload['sub']
                except DecodeError:
                    print "Decode error: Could not decode authorization header."
                    g.user_id = None
                    payload = None
                except ExpiredSignature:
                    print "Expired signature error: Signature expired."
                    g.user_id = None
                    payload = None
                if payload and datetime.fromtimestamp(payload['exp']) < datetime.now():
                    g.user_id = None

            return f(*args, **kwargs)

        return decorated_function

    def sameIP_required(self, f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if request.remote_addr != '127.0.0.1':
                response = jsonify(message="No access")
                response.status_code = 401
                return response
            return f(*args, **kwargs)

        return decorated_function


myWrappers = MyWrappers()
