"""remove unused columns

Revision ID: 18c777926a6b
Revises: fac59f85375
Create Date: 2015-05-13 09:55:28.266648

"""

# revision identifiers, used by Alembic.
revision = '18c777926a6b'
down_revision = 'fac59f85375'

import sqlalchemy as sa
from alembic import op


def drop_column_sqlite(tablename, columns):
    """ column dropping functionality for SQLite """

    # we need copy to make a deep copy of the column attributes
    from copy import copy

    # get the db engine and reflect database tables
    engine = op.get_bind()
    meta = sa.MetaData(bind=engine)
    meta.reflect()

    # create a select statement from the old table
    old_table = meta.tables[tablename]
    select = sa.sql.select([c for c in old_table.c if c.name not in columns])

    # get remaining columns without table attribute attached
    remaining_columns = [copy(c) for c in old_table.columns
            if c.name not in columns]
    for column in remaining_columns:
        column.table = None

    # create a temporary new table
    new_tablename = '{0}_new'.format(tablename)
    op.create_table(new_tablename, *remaining_columns)
    meta.reflect()
    new_table = meta.tables[new_tablename]

    # copy data from old table
    insert = sa.sql.insert(new_table).from_select(
            [c.name for c in remaining_columns], select)
    engine.execute(insert)

    # drop the old table and rename the new table to take the old tables
    # position
    op.drop_table(tablename)
    op.rename_table(new_tablename, tablename)


def drop_column(tablename, columnname):
    drop_column_sqlite(tablename, [columnname])

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    drop_column('problems', 'output_file')
    drop_column('problems', 'deadline')
    drop_column('problems', 'img')
    drop_column('problems', 'input_file')
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('problems', sa.Column('input_file', sa.VARCHAR(length=120), nullable=True))
    op.add_column('problems', sa.Column('img', sa.VARCHAR(length=120), nullable=True))
    op.add_column('problems', sa.Column('deadline', sa.DATETIME(), nullable=True))
    op.add_column('problems', sa.Column('output_file', sa.VARCHAR(length=120), nullable=True))
    ### end Alembic commands ###
