import os
import json
import datetime

import requests

from subprocess32 import TimeoutExpired, CalledProcessError

from cmb_utils.checker import CheckerError
from cmb_utils.execute_on_backend import ExecuteOnBackend

SERVER_TIMEOUT = int(os.environ['SERVER_TIMEOUT'])


class Push():
    dequeue_url = 'http://localhost:5000/api/dequeue'
    result_url = 'http://localhost:5000/api/result'
    profiling_url = 'http://localhost:5000/api/profiling'

    @staticmethod
    def get_object_from_run_queue():
        r = requests.get(Push.dequeue_url)
        return r.json()

    @staticmethod
    def get_current_time_string():
        return datetime.datetime.now().strftime("%I:%M:%S%p on %B %d, %Y")

    @staticmethod
    def run():
        dateString = Push.get_current_time_string()
        obj = Push.get_object_from_run_queue()
        filename = obj['filename']

        if not filename:
            return

        problemName = obj['problemName']
        userName = obj['userName']

        logString = dateString + ": Filename found in queue: " + filename + " for problem [" + problemName + "]"
        logString += "\nSubmitted by user: " + userName
        print(logString)

        # Try run the submitted program
        dict_results = {'isCorrect': False, 'submissionId': obj['submissionId'], 'runId': obj['runId']}
        if obj['profiling']:  # Do a profiling run
            dict_results.update(ExecuteOnBackend.profiling(problemName, userName, filename))
            print('Done profiling', filename)
            print(dict_results)
            return requests.put(Push.profiling_url, data=json.dumps(dict_results))

        try:
            dict_results.update(ExecuteOnBackend.run(problemName, userName, filename))
        except TimeoutExpired as e:
            print(e)
            dict_results["msg"] = 'Your run took too long to execute and was stopped due ' \
                                  'to a timeout at the server. Please check your code and resubmit.'
            dict_results["state"] = "SERVER_TIMEOUT"
        except CheckerError as e:
            print(e)
            dict_results['msg'] = 'Failed big correction test'
        except CalledProcessError as e:
            print(e)
            dict_results['msg'] = 'An error occurred during your run. Please notify the ' \
                                  'system administrators that yor run failed.'
        except Exception as e:
            print(e)
            # Most likely caused by a non-zero return code
            dict_results['msg'] = 'Please notify the system administrators that you received this error message.'

        # Try load the output dict_results
        try:
            requests.put(Push.result_url, data=json.dumps(dict_results))
        except Exception as e:
            print(e)
            print("Error in " + filename)
            print(dict_results)
            outputString = userName + " had a problem with the output on submission "
            outputString += filename + " on " + problemName + ". Tell administrators to check your "
            outputString += " submission for the specific error."
            dict_results['isCorrect'] = False
            dict_results['msg'] = outputString
            requests.put(Push.result_url, data=json.dumps(dict_results))
        print("Done running", filename)


if __name__ == "__main__":
    P = Push()
    P.run()
