from flask import Blueprint, g, jsonify

from cmb_utils.helpers import problem_to_json, submission_to_json
from cmb_utils.wrappers import myWrappers
from database.models import Problem, User
from routes.submissions import has_flame_graph

problem_routes = Blueprint('problem_routes', __name__, None)


@problem_routes.route('/api/problems', methods=['GET'])
def get_problems():
    problems = Problem.query.filter_by(visible=True).all()
    return jsonify({'problems': map(problem_to_json, problems)})


@problem_routes.route('/api/problem/<int:problem_id>', methods=['GET'])
def get_problem(problem_id):
    problem = Problem.query.filter_by(id=problem_id, visible=True).first()
    if problem:
        return jsonify({'problem': problem_to_json(problem)})
    else:
        return jsonify({'message': 'Problem not available'})


@problem_routes.route('/api/submissions/<int:problem_id>', methods=['GET'])
@myWrappers.login_required
def get_submissions(problem_id):
    user = User.query.filter_by(id=g.user_id).first()
    submissions = []
    for sub in user.submissions:
        if sub.problem_id == problem_id:
            submissions.append(submission_to_json(sub))
    return jsonify({'submissions': submissions})
