from flask.ext.testing import TestCase

from database.models import AdminUser
from server import create_app, db


class problemsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()
        db.session.commit()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def login(self, username, password):
        return self.client.post("admin/login", data=dict(
            username=username,
            password=password
        ), follow_redirects=True)

    def logout(self):
        return self.client.get("admin/logout", follow_redirects=True)

    def test_login_logout(self):
        adminUser = AdminUser("testAdmin", "adminPW")
        db.session.add(adminUser)
        db.session.commit()
        response = self.login("testAdmin", "adminPW")
        self.assert200(response)
        assert "You are logged in!" in response.data
        response = self.logout()
        self.assert200(response)
        assert "Login" in response.data
        response = self.login("wrongName", "adminPW")
        self.assert200(response)
        assert "Invalid user" in response.data
        response = self.login("testAdmin", "wrongPW")
        self.assert200(response)
        assert "Invalid password" in response.data

    def test_access_denied(self):
        response = self.client.get("/admin/user/")
        self.assert403(response)
        response = self.client.get("/admin/problem/")
        self.assert403(response)
        response = self.client.get("/admin/submission/")
        self.assert403(response)
        response = self.client.get("/admin/uploads/")
        self.assert403(response)
        response = self.client.get("/admin/bulletin/")
        self.assert403(response)

    def test_access_allowed(self):
        adminUser = AdminUser("testAdmin", "adminPW")
        db.session.add(adminUser)
        db.session.commit()
        response = self.login("testAdmin", "adminPW")
        self.assert200(response)
        response = self.client.get("/admin/user/")
        self.assert200(response)
        response = self.client.get("/admin/problem/")
        self.assert200(response)
        response = self.client.get("/admin/submission/")
        self.assert200(response)
        response = self.client.get("/admin/uploads/")
        self.assert200(response)
        response = self.client.get("/admin/bulletin/")
        self.assert200(response)
