from flask.ext.testing import TestCase
from mock import patch

from database.models import Bulletin
from server import create_app, db

from datetime import datetime, timedelta


class problemsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()
        db.session.commit()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_active_bulletins(self):
        with patch('routes.bulletin.get_current_time') as MockClass:
            current_time = datetime(2016, 1, 3)
            MockClass.return_value = current_time

            bulletin = Bulletin(content='some content', removable=True, valid_until=current_time+timedelta(2))
            bulletin2 = Bulletin(content='some content', removable=True, valid_until=current_time-timedelta(2))

            db.session.add(bulletin)
            db.session.add(bulletin2)
            db.session.commit()

            bulletins = Bulletin.query.filter(Bulletin.valid_until > current_time).first()
            response = self.client.get("api/bulletins/active")
            expected = dict(bulletins=[
                            {'content': bulletins.content,
                             'valid_until': bulletins.valid_until.strftime("%a, %d %b %Y %M:%M:%S GMT"),
                             'id': bulletins.id, 'removable': bulletins.removable}
                            ])

            self.assertEquals(response.json, expected)

            bulletins = Bulletin.query.all()
            response = self.client.get("api/bulletins")
            expected = dict(bulletins=[
                            {'content': bulletins[0].content,
                             'valid_until': bulletins[0].valid_until.strftime("%a, %d %b %Y %M:%M:%S GMT"),
                             'id': bulletins[0].id, 'removable': bulletins[0].removable},
                            {'content': bulletins[1].content,
                             'valid_until': bulletins[1].valid_until.strftime("%a, %d %b %Y %M:%M:%S GMT"),
                             'id': bulletins[1].id, 'removable': bulletins[1].removable}])
            self.assertEquals(response.json, expected)

    def test_no_bulletins(self):
        response = self.client.get("api/bulletins/active")
        self.assert404(response)
        self.assertEquals(response.json, dict(message='No active bulletins.'))
        response = self.client.get("api/bulletins")
        self.assert404(response)
        self.assertEquals(response.json, dict(message='No bulletins.'))
