import os
import shutil

from flask.ext.testing import TestCase
from mock import patch

from cmb_utils.compilation_check import CompilationError, CompilationCheck, delete_directory, join_paths

from server import create_app, db


correctCPPProgram = "int main() { return 0; }"
incorrectCPPProgram = "this should not compile"


class checkCompilationTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        return tmp

    def test_join_paths(self):
        source_path = os.path.abspath('.')
        this_path = os.path.realpath(__file__)
        joined_path = join_paths([source_path, 'tests', 'unit', 'compilation_check_test.py'])
        self.assertTrue(this_path == joined_path or this_path == joined_path + 'c')

    def test_delete_directory(self):
        test_dir = os.path.join(os.path.abspath('.'), 'delete_me')
        os.mkdir(test_dir)
        delete_directory(test_dir)
        self.assertFalse(os.path.exists(test_dir))

    def test_check_compilation_correct(self):
        flask_dir = os.path.abspath('..')
        mali_dir = os.path.abspath('../../../Mali_OpenCL_SDK_v1.1.0')
        test_folder_dir = os.path.join(flask_dir, 'problems/test_problem/uploads/test_user_name/correct_program')
        try:
            os.makedirs(test_folder_dir)
        except OSError:
            pass
        with open(os.path.join(test_folder_dir, 'correct_program.cpp'), 'w') as cpp_file:
            cpp_file.write(correctCPPProgram)

        shutil.make_archive(test_folder_dir, 'zip', os.path.dirname(test_folder_dir), 'correct_program')

        try:
            CompilationCheck.run('test_problem', 'test_user_name', 'correct_program.zip', 0, flask_dir, mali_dir)
        except CompilationError as e:
            self.fail(str(e))

        shutil.rmtree(os.path.join(flask_dir, 'problems/test_problem'))

    def test_check_compilation_incorrect(self):
        flask_dir = os.path.abspath('..')
        mali_dir = os.path.abspath('../../../Mali_OpenCL_SDK_v1.1.0')
        test_folder_dir = os.path.join(flask_dir, 'problems/test_problem/uploads/test_user_name/incorrect_program')
        try:
            os.makedirs(test_folder_dir)
        except OSError:
            pass
        with open(os.path.join(test_folder_dir, 'incorrect_program.cpp'), 'w') as cpp_file:
            cpp_file.write(incorrectCPPProgram)

        shutil.make_archive(test_folder_dir, 'zip', os.path.dirname(test_folder_dir), 'incorrect_program')

        try:
            CompilationCheck.run('test_problem', 'test_user_name', 'incorrect_program.zip', 0,
                                                 flask_dir, mali_dir)
            self.fail("Didn't raise exception when it should.")
        except CompilationError as e:
            self.assertEquals(str(e), 'Compilation failed with exit code: 2')

        shutil.rmtree(os.path.join(flask_dir, 'problems/test_problem'))

