import os
import shutil

import subprocess32
from flask.ext.testing import TestCase
from mock import patch

from cmb_utils.execute_on_backend import ExecuteOnBackend
from cmb_utils.checker import Checker

from server import create_app

correct_solution = '#include<iostream>\n' \
                   'int main() {\n' \
                   'std::cout << "Hello Italy!" << std::endl;\n' \
                   'std::cout << "Greetings from the CMB-team at NTNU in Trondheim, Norway.";\n' \
                   '}'
incorrect_solution = '#include<iostream>\n' \
                     'int main() {\n' \
                     'std::cout << "Hely!";\n' \
                     '}'


class executeOnBackendTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        return tmp

    def test_board_down(self):
        execute_on_backend = ExecuteOnBackend()
        execute_on_backend.board_ip = 'x.x.x.x.x.x.x.x.x'
        self.assertRaises(Exception, execute_on_backend.ping_board)

    def test_execute_correct(self):
        flask_dir = os.path.abspath('..')
        hello_italy_io_dir = os.path.join(flask_dir, 'problems/hello_italy/problemIO')
        self.assertTrue(os.path.exists(hello_italy_io_dir), 'Test requires hello italy problem.')

        test_submission_dir = os.path.join(flask_dir, 'problems/hello_italy/uploads/test_user_name/correct_program')
        try:
            os.makedirs(test_submission_dir)
        except OSError:
            pass
        with open(os.path.join(test_submission_dir, 'correct_program.cpp'), 'w') as cpp_file:
            cpp_file.write(correct_solution)

        with patch.object(ExecuteOnBackend, 'ping_board') as ping_board, \
                patch.object(ExecuteOnBackend, 'run_script_on_board') as run_script_on_board, \
                patch.object(ExecuteOnBackend, 'copy_file_to_server') as copy_file_to_server, \
                patch.object(ExecuteOnBackend, 'copy_files_to_board') as copy_files_to_board, \
                patch.object(Checker, 'run') as run_checker:
            ping_board.return_value = None
            copy_file_to_server.return_value = None
            copy_files_to_board.return_value = None
            run_script_on_board.return_value = '1\n/tmp/fileR2gGUP\n{"time":0.05, "power":0, "energy":0.11034}'
            run_checker.return_value = True, 123

            values = ExecuteOnBackend.run('hello_italy', 'test_user_name', 'correct_program.zip')

            self.assertTrue(values["isCorrect"])
            self.assertEquals(values["goodness"], 123)

        shutil.rmtree(os.path.join(flask_dir, 'problems/hello_italy/uploads/test_user_name'))

    def test_execute_incorrect(self):
        flask_dir = os.path.abspath('..')
        hello_italy_io_dir = os.path.join(flask_dir, 'problems/hello_italy/problemIO')
        self.assertTrue(os.path.exists(hello_italy_io_dir), 'Test requires hello italy problem.')

        test_submission_dir = os.path.join(flask_dir, 'problems/hello_italy/uploads/test_user_name/incorrect_program')
        try:
            os.makedirs(test_submission_dir)
        except OSError:
            pass
        with open(os.path.join(test_submission_dir, 'incorrect_program.cpp'), 'w') as cpp_file:
            cpp_file.write(incorrect_solution)

        with patch.object(ExecuteOnBackend, 'ping_board') as ping_board, \
                patch.object(ExecuteOnBackend, 'run_script_on_board') as run_script_on_board, \
                patch.object(ExecuteOnBackend, 'copy_file_to_server') as copy_file_to_server, \
                patch.object(ExecuteOnBackend, 'copy_files_to_board') as copy_files_to_board, \
                patch.object(Checker, 'run') as run_checker:
            ping_board.return_value = None
            copy_file_to_server.return_value = None
            copy_files_to_board.return_value = None
            run_script_on_board.return_value = '0\n{"isCorrect":false, "msg":"Failed correctness test! Result: Hello Italy! Greetings from the CMB-team at NTNU in Trondheim, Norway. was: Hel ndheim, Norway.", "state": "CORRECTNESS_BAD_OUTPUT"}'
            run_checker.return_value = True, "null"

            values = ExecuteOnBackend.run('hello_italy', 'test_user_name', 'incorrect_program.zip')

            self.assertFalse(values["isCorrect"])

        shutil.rmtree(os.path.join(flask_dir, 'problems/hello_italy/uploads/test_user_name'))
