from flask.ext.testing import TestCase

from cmb_utils.helpers import (create_jwt_token, problem_to_json,
                               submission_to_json)
from database.models import Problem, Submission, User
from server import create_app, db


class problemsTest(TestCase):
    Testing = True

    def create_app(self):
        tmp = create_app(self, push=False)
        # tmp.config['SERVER_PORT']=1337
        tmp.config['Testing'] = True
        tmp.config['SQLALCHEMY_DATABASE_URI'] = "sqlite:///tests/testDB"
        return tmp

    def setUp(self):
        db.create_all()
        problem = Problem("Test", "test")
        problem2 = Problem("Test2", "test2")
        problem2.visible = False
        db.session.add(problem)
        db.session.add(problem2)
        db.session.commit()

    def tearDown(self):
        db.session.remove()
        db.drop_all()

    def test_get_problems_route(self):
        prob = Problem.query.filter_by(id=1).first()
        response = self.client.get("/api/problems")
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(problems=[problem_to_json(prob)]))

    def test_get_problem_route(self):
        self.maxDiff = None
        prob = Problem.query.filter_by(id=1).first()
        response = self.client.get("/api/problem/1")
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(problem=problem_to_json(prob)))

    def test_get_users_submissions_to_problem_route(self):
        problem2 = Problem("Test2", "test2")
        db.session.add(problem2)
        user = User(name="Test", password="test")
        user2 = User(name="Test2", password="test2")
        db.session.add(user)
        db.session.add(user2)
        db.session.commit()
        sub = Submission("filename", 1, 2, "Uploaded")
        sub2 = Submission("filename2", 2, 1, "Uploaded")
        db.session.add(sub)
        db.session.add(sub2)
        db.session.commit()

        # get all submissions user Test2 for problem 1
        test_sub = Submission.query.filter_by(name="filename").first()
        test_sub2 = Submission.query.filter_by(name="filename2").first()
        token = create_jwt_token(self.app, user2)
        response = self.client.get("/api/submissions/1", headers=[('Authorization', 'Bearer ' + token)])
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(submissions=[submission_to_json(test_sub)]))
        # get all submissions for user Test for problem 2
        token2 = create_jwt_token(self.app, user)
        response = self.client.get("/api/submissions/2", headers=[('Authorization', 'Bearer ' + token2)])
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.json, dict(submissions=[submission_to_json(test_sub2)]))
